@@ -1,32 +1,12 @@
-/*
- * Copyright (C) 2014 - 2019 Intel Corporation.
- * All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions are met:
- * 1. Redistributions of source code must retain the above copyright notice(s),
- *    this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright notice(s),
- *    this list of conditions and the following disclaimer in the documentation
- *    and/or other materials provided with the distribution.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER(S) ``AS IS'' AND ANY EXPRESS
- * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
- * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO
- * EVENT SHALL THE COPYRIGHT HOLDER(S) BE LIABLE FOR ANY DIRECT, INDIRECT,
- * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
- * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
- * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
- * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
- * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- */
+// SPDX-License-Identifier: BSD-2-Clause
+/* Copyright (C) 2014 - 2021 Intel Corporation. */
 
 #pragma once
 #ifdef __cplusplus
 extern "C" {
 #endif
 
+#include <stdbool.h>
 #include <sys/types.h>
 
 /**
@@ -41,8 +21,8 @@ extern "C" {
 
 /// \brief Memkind memory types
 /// \warning EXPERIMENTAL API
-typedef enum memkind_memtype_t {
-
+typedef enum memkind_memtype_t
+{
     /**
      * Select standard memory, the same as process use.
      */
@@ -54,15 +34,14 @@ typedef enum memkind_memtype_t {
      * determine the HBM.
      */
     MEMKIND_MEMTYPE_HIGH_BANDWIDTH = _MEMKIND_BIT(1)
-
 } memkind_memtype_t;
 
 #undef _MEMKIND_BIT
 
 /// \brief Memkind policy
 /// \warning EXPERIMENTAL API
-typedef enum memkind_policy_t {
-
+typedef enum memkind_policy_t
+{
     /**
      * Allocate local memory.
      * If there is not enough memory to satisfy the request errno is set to
@@ -103,66 +82,221 @@ typedef enum memkind_policy_t {
      * Max policy value.
      */
     MEMKIND_POLICY_MAX_VALUE
-
 } memkind_policy_t;
 
 /// \brief Memkind bits definition
 /// \warning EXPERIMENTAL API
-/// \note The bits specify flags and masks. Bits <0,1,2,...,7> are reserved for page size, where page sizes are encoded
-///       by base-2 logarithm. If the page size bits are set to zero value, than default page size will be used.
-typedef enum memkind_bits_t {
-    MEMKIND_MASK_PAGE_SIZE_2MB = 21ull,  /**<  Allocations backed by 2 MB page size (2^21 = 2MB) */
+/// \note The bits specify flags and masks. Bits <0,1,2,...,7> are reserved for
+///       page size, where page sizes are encoded by base-2 logarithm. If the
+///       page size bits are set to zero value, than default page size will be
+///       used.
+typedef enum memkind_bits_t
+{
+    /**
+     * Allocations backed by 2 MB page size (2^21 = 2MB).
+     */
+    MEMKIND_MASK_PAGE_SIZE_2MB = 21ull
 } memkind_bits_t;
 
 /// \brief Memkind type definition
 /// \warning EXPERIMENTAL API
 typedef struct memkind *memkind_t;
 
-
 /// \brief Memkind constant values
 /// \warning EXPERIMENTAL API
-enum memkind_const {
-    MEMKIND_MAX_KIND = 512,                     /**<  Maximum number of kinds */
-    MEMKIND_ERROR_MESSAGE_SIZE = 128,           /**<  Error message size */
-    MEMKIND_PMEM_MIN_SIZE = (1024 * 1024 * 16)  /**<  The minimum size which allows to limit the file-backed memory partition */
+enum memkind_const
+{
+    /**
+     * Maximum number of kinds.
+     */
+    MEMKIND_MAX_KIND = 512,
+
+    /**
+     * Error message size.
+     */
+    MEMKIND_ERROR_MESSAGE_SIZE = 128,
+
+    /**
+     * The minimum size which allows to limit the file-backed memory partition.
+     */
+    MEMKIND_PMEM_MIN_SIZE = (1024 * 1024 * 16)
 };
 
 /// \brief Memkind operation statuses
 /// \warning EXPERIMENTAL API
-enum {
-    MEMKIND_SUCCESS = 0,                        /**<  Operation success */
-    MEMKIND_ERROR_UNAVAILABLE = -1,             /**<  Error: Memory kind is not available */
-    MEMKIND_ERROR_MBIND = -2,                   /**<  Error: Call to mbind() failed */
-    MEMKIND_ERROR_MMAP  = -3,                   /**<  Error: Call to mmap() failed */
-    MEMKIND_ERROR_MALLOC = -6,                  /**<  Error: Call to malloc() failed */
-    MEMKIND_ERROR_ENVIRON = -12,                /**<  Error: Unable to parse environment variable */
-    MEMKIND_ERROR_INVALID = -13,                /**<  Error: Invalid argument */
-    MEMKIND_ERROR_TOOMANY = -15,                /**<  Error: Attempt to initialize more than MEMKIND_MAX_KIND number of kinds */
-    MEMKIND_ERROR_BADOPS = -17,                 /**<  Error: Invalid memkind_ops structure */
-    MEMKIND_ERROR_HUGETLB = -18,                /**<  Error: Unable to allocate huge pages */
-    MEMKIND_ERROR_MEMTYPE_NOT_AVAILABLE = -20,  /**<  Error: Requested memory type is not available */
-    MEMKIND_ERROR_OPERATION_FAILED = -21,       /**<  Error: Operation failed */
-    MEMKIND_ERROR_ARENAS_CREATE = -22,          /**<  Error: Call to jemalloc's arenas.create failed */
-    MEMKIND_ERROR_RUNTIME = -255                /**<  Error: Unspecified run-time error */
+enum
+{
+    /**
+     * Operation success.
+     */
+    MEMKIND_SUCCESS = 0,
+
+    /**
+     * Error: Memory kind is not available.
+     */
+    MEMKIND_ERROR_UNAVAILABLE = -1,
+
+    /**
+     * Error: Call to mbind() failed.
+     */
+    MEMKIND_ERROR_MBIND = -2,
+
+    /**
+     * Error: Call to mmap() failed.
+     */
+    MEMKIND_ERROR_MMAP = -3,
+
+    /**
+     * Error: Call to malloc() failed.
+     */
+    MEMKIND_ERROR_MALLOC = -6,
+
+    /**
+     * Error: Unable to parse environment variable.
+     */
+    MEMKIND_ERROR_ENVIRON = -12,
+
+    /**
+     * Error: Invalid argument.
+     */
+    MEMKIND_ERROR_INVALID = -13,
+
+    /**
+     * Error: Attempt to initialize more than MEMKIND_MAX_KIND number of kinds.
+     */
+    MEMKIND_ERROR_TOOMANY = -15,
+
+    /**
+     * Error: Invalid memkind_ops structure.
+     */
+    MEMKIND_ERROR_BADOPS = -17,
+
+    /**
+     * Error: Unable to allocate huge pages.
+     */
+    MEMKIND_ERROR_HUGETLB = -18,
+
+    /**
+     * Error: Requested memory type is not available.
+     */
+    MEMKIND_ERROR_MEMTYPE_NOT_AVAILABLE = -20,
+
+    /**
+     * Error: Operation failed.
+     */
+    MEMKIND_ERROR_OPERATION_FAILED = -21,
+
+    /**
+     * Error: Call to jemalloc's arenas.create failed.
+     */
+    MEMKIND_ERROR_ARENAS_CREATE = -22,
+
+    /**
+     * Error: Unspecified run-time error.
+     */
+    MEMKIND_ERROR_RUNTIME = -255
 };
 
 /* KIND CONFIGURATION MANAGEMENT INTERFACE */
 
 /// \brief Memkind memory usage policy
-typedef enum memkind_mem_usage_policy {
-    MEMKIND_MEM_USAGE_POLICY_DEFAULT      = 0,        /**<  Default memory usage */
-    MEMKIND_MEM_USAGE_POLICY_CONSERVATIVE = 1,        /**<  Minimize memory usage at all costs, */
+typedef enum memkind_mem_usage_policy
+{
+    /**
+     * Default memory usage.
+     */
+    MEMKIND_MEM_USAGE_POLICY_DEFAULT = 0,
+
+    /**
+     * Minimize memory usage at all costs.
+     */
+    MEMKIND_MEM_USAGE_POLICY_CONSERVATIVE = 1,
+
+    /**
+     * Max memory usage policy value.
+     */
     MEMKIND_MEM_USAGE_POLICY_MAX_VALUE
 } memkind_mem_usage_policy;
 
 /// \brief Memkind memory statistics type
-typedef enum memkind_stat_type {
-    MEMKIND_STAT_TYPE_RESIDENT      = 0,        /**<  Maximum number of bytes in physically resident data pages mapped */
-    MEMKIND_STAT_TYPE_ACTIVE        = 1,        /**<  Total number of bytes in active pages */
-    MEMKIND_STAT_TYPE_ALLOCATED     = 2,        /**<  Total number of allocated bytes */
+typedef enum memkind_stat_type
+{
+    /**
+     * Maximum number of bytes in physically resident data pages mapped.
+     */
+    MEMKIND_STAT_TYPE_RESIDENT = 0,
+
+    /**
+     * Total number of bytes in active pages.
+     */
+    MEMKIND_STAT_TYPE_ACTIVE = 1,
+
+    /**
+     * Total number of allocated bytes.
+     */
+    MEMKIND_STAT_TYPE_ALLOCATED = 2,
+
+    /**
+     * Max memory statistics type.
+     */
     MEMKIND_STAT_TYPE_MAX_VALUE
 } memkind_stat_type;
 
+/// \brief Memory statistics print options
+typedef enum memkind_stat_print_opt
+{
+
+    /**
+     * Print all stats
+     */
+    MEMKIND_STAT_PRINT_ALL = 0,
+
+    /**
+     * Print stats in JSON format
+     */
+    MEMKIND_STAT_PRINT_JSON_FORMAT = 1U << 0,
+
+    /**
+     * Omit general information that never changes during execution
+     */
+    MEMKIND_STAT_PRINT_OMIT_GENERAL = 1U << 1,
+
+    /**
+     * Omit merged arena statistics
+     */
+    MEMKIND_STAT_PRINT_OMIT_MERGED_ARENA = 1U << 2,
+
+    /**
+     * Omit destroyed merged arena statistics
+     */
+    MEMKIND_STAT_PRINT_OMIT_DESTROYED_MERGED_ARENA = 1U << 3,
+
+    /**
+     * Omit per arena statistics
+     */
+    MEMKIND_STAT_PRINT_OMIT_PER_ARENA = 1U << 4,
+
+    /**
+     * Omit per size class statistics for bins
+     */
+    MEMKIND_STAT_PRINT_OMIT_PER_SIZE_CLASS_BINS = 1U << 5,
+
+    /**
+     * Omit per size class statistics for large objects
+     */
+    MEMKIND_STAT_PRINT_OMIT_PER_SIZE_CLASS_LARGE = 1U << 6,
+
+    /**
+     * Omit all mutex statistics
+     */
+    MEMKIND_STAT_PRINT_OMIT_MUTEX = 1U << 7,
+
+    /**
+     * Omit extent statistics
+     */
+    MEMKIND_STAT_PRINT_OMIT_EXTENT = 1U << 8,
+} memkind_stat_print_opt;
+
 /// \brief Forward declaration of memkind configuration
 struct memkind_config;
 
@@ -181,7 +315,8 @@ struct memkind_config *memkind_config_new(void);
 void memkind_config_delete(struct memkind_config *cfg);
 
 ///
-/// \brief Update memkind configuration with path to specified directory parameter
+/// \brief Update memkind configuration with path to specified directory
+///        parameter
 /// \note STANDARD API
 /// \param cfg memkind configuration
 /// \param pmem_dir path to specified directory for PMEM kind
@@ -206,26 +341,33 @@ void memkind_config_set_memory_usage_policy(struct memkind_config *cfg,
                                             memkind_mem_usage_policy policy);
 
 ///
-/// \brief Create kind that allocates memory with specific memory type, memory binding policy and flags.
+/// \brief Create kind that allocates memory with specific memory type, memory
+///        binding policy and flags.
 /// \warning EXPERIMENTAL API
 /// \note Currently implemented memory type and policy configurations:
-///.      {MEMKIND_MEMTYPE_DEFAULT, MEMKIND_POLICY_PREFERRED_LOCAL},
-///.      {MEMKIND_MEMTYPE_HIGH_BANDWIDTH, MEMKIND_POLICY_BIND_LOCAL},
+///       {MEMKIND_MEMTYPE_DEFAULT, MEMKIND_POLICY_PREFERRED_LOCAL},
+///       {MEMKIND_MEMTYPE_HIGH_BANDWIDTH, MEMKIND_POLICY_BIND_LOCAL},
 ///       {MEMKIND_MEMTYPE_HIGH_BANDWIDTH, MEMKIND_POLICY_PREFERRED_LOCAL},
 ///       {MEMKIND_MEMTYPE_HIGH_BANDWIDTH, MEMKIND_POLICY_INTERLEAVE_ALL},
-///       {MEMKIND_MEMTYPE_DEFAULT | MEMKIND_MEMTYPE_HIGH_BANDWIDTH, MEMKIND_POLICY_INTERLEAVE_ALL}.
-/// \param memtype_flags determine the memory types to allocate from by combination of memkind_memtype_t values.
-///        This field cannot have zero value.
-/// \param policy specify policy for page binding to memory types selected by  memtype_flags.
-///        This field must be set to memkind_policy_t value. If policy is set to MEMKIND_POLICY_PREFERRED_LOCAL then only one memory
-///        type must be selected. Note: the value cannot be set to MEMKIND_POLICY_MAX_VALUE.
-/// \param flags the field must be set to a combination of memkind_bits_t values.
+///       {MEMKIND_MEMTYPE_DEFAULT | MEMKIND_MEMTYPE_HIGH_BANDWIDTH,
+///       MEMKIND_POLICY_INTERLEAVE_ALL}.
+/// \param memtype_flags determine the memory types to allocate from by
+///        combination of memkind_memtype_t values. This field cannot have zero
+///        value.
+/// \param policy specify policy for page binding to memory types selected by
+///        memtype_flags. This field must be set to memkind_policy_t value. If
+///        policy is set to MEMKIND_POLICY_PREFERRED_LOCAL then only one memory
+///        type must be selected. Note: the value cannot be set to
+///        MEMKIND_POLICY_MAX_VALUE.
+/// \param flags the field must be set to a combination of memkind_bits_t
+///        values.
 /// \param kind pointer to kind which will be created
-/// \return Memkind operation status, MEMKIND_SUCCESS on success, MEMKIND_ERROR_MEMTYPE_NOT_AVAILABLE or MEMKIND_ERROR_INVALID on failure
+/// \return Memkind operation status, MEMKIND_SUCCESS on success,
+///         MEMKIND_ERROR_MEMTYPE_NOT_AVAILABLE or MEMKIND_ERROR_INVALID on
+///         failure
 ///
 int memkind_create_kind(memkind_memtype_t memtype_flags,
-                        memkind_policy_t policy,
-                        memkind_bits_t flags,
+                        memkind_policy_t policy, memkind_bits_t flags,
                         memkind_t *kind);
 
 ///
@@ -234,10 +376,12 @@ int memkind_create_kind(memkind_memtype_t memtype_flags,
 ///        The function has undefined behavior when the handle is invalid or
 ///        memkind_destroy_kind(kind) was already called before
 /// \warning EXPERIMENTAL API
-/// \note if the kind was returned by memkind_create_kind() all allocated memory must be freed
+/// \note if the kind was returned by memkind_create_kind() all allocated memory
+///       must be freed
 ///       before kind is destroyed, otherwise this will cause memory leak.
 /// \param kind specified memory kind
-/// \return Memkind operation status, MEMKIND_SUCCESS on success, MEMKIND_ERROR_OPERATION_FAILED on failure
+/// \return Memkind operation status, MEMKIND_SUCCESS on success,
+///         MEMKIND_ERROR_OPERATION_FAILED on failure
 ///
 int memkind_destroy_kind(memkind_t kind);
 
@@ -294,10 +438,38 @@ extern memkind_t MEMKIND_DAX_KMEM_ALL;
 /// \note STANDARD API
 extern memkind_t MEMKIND_DAX_KMEM_PREFERRED;
 
+/// \note STANDARD API
+extern memkind_t MEMKIND_DAX_KMEM_INTERLEAVE;
+
+/// \note STANDARD API
+extern memkind_t MEMKIND_HIGHEST_CAPACITY;
+
+/// \note STANDARD API
+extern memkind_t MEMKIND_HIGHEST_CAPACITY_PREFERRED;
+
+/// \note STANDARD API
+extern memkind_t MEMKIND_HIGHEST_CAPACITY_LOCAL;
+
+/// \note STANDARD API
+extern memkind_t MEMKIND_HIGHEST_CAPACITY_LOCAL_PREFERRED;
+
+/// \note STANDARD API
+extern memkind_t MEMKIND_LOWEST_LATENCY_LOCAL;
+
+/// \note STANDARD API
+extern memkind_t MEMKIND_LOWEST_LATENCY_LOCAL_PREFERRED;
+
+/// \note STANDARD API
+extern memkind_t MEMKIND_HIGHEST_BANDWIDTH_LOCAL;
+
+/// \note STANDARD API
+extern memkind_t MEMKIND_HIGHEST_BANDWIDTH_LOCAL_PREFERRED;
+
 ///
 /// \brief Get Memkind API version
 /// \note STANDARD API
-/// \return Version number represented by a single integer number(major * 1000000 + minor * 1000 + patch)
+/// \return Version number represented by a single integer number(major *
+///         1000000 + minor * 1000 + patch)
 ///
 int memkind_get_version();
 
@@ -311,13 +483,14 @@ int memkind_get_version();
 void memkind_error_message(int err, char *msg, size_t size);
 
 ///
-/// \brief Create a new PMEM (file-backed) kind of given size on top of a temporary file
-///        in the given directory dir
+/// \brief Create a new PMEM (file-backed) kind of given size on top of a
+///        temporary file in the given directory dir
 /// \note STANDARD API
 /// \param dir path to specified directory to temporary file
 /// \param max_size size limit for kind
 /// \param kind pointer to kind which will be created
-/// \return Memkind operation status, MEMKIND_SUCCESS on success, other values on failure
+/// \return Memkind operation status, MEMKIND_SUCCESS on success, other values
+///         on failure
 ///
 int memkind_create_pmem(const char *dir, size_t max_size, memkind_t *kind);
 
@@ -326,7 +499,8 @@ int memkind_create_pmem(const char *dir, size_t max_size, memkind_t *kind);
 /// \note STANDARD API
 /// \param cfg memkind configuration for specifying PMEM parameters
 /// \param kind pointer to kind which will be created
-/// \return Memkind operation status, MEMKIND_SUCCESS on success, other values on failure
+/// \return Memkind operation status, MEMKIND_SUCCESS on success, other values
+///         on failure
 ///
 int memkind_create_pmem_with_config(struct memkind_config *cfg,
                                     memkind_t *kind);
@@ -335,14 +509,16 @@ int memkind_create_pmem_with_config(struct memkind_config *cfg,
 /// \brief Check if kind is available
 /// \note STANDARD API
 /// \param kind specified memory kind
-/// \return Memkind operation status, MEMKIND_SUCCESS on success, other values on failure
+/// \return Memkind operation status, MEMKIND_SUCCESS on success, other values
+///         on failure
 ///
 int memkind_check_available(memkind_t kind);
 
 ///
 /// \brief Update memkind cached statistics
 /// \note STANDARD API
-/// \return Memkind operation status, MEMKIND_SUCCESS on success, other values on failure
+/// \return Memkind operation status, MEMKIND_SUCCESS on success, other values
+///         on failure
 ///
 int memkind_update_cached_stats(void);
 
@@ -352,10 +528,24 @@ int memkind_update_cached_stats(void);
 /// \param kind specified memory kind
 /// \param stat specified type of memory statistic
 /// \param value reference to value of memory statistic
-/// \return Memkind operation status, MEMKIND_SUCCESS on success, other values on failure
+/// \return Memkind operation status, MEMKIND_SUCCESS on success, other values
+///         on failure
 ///
 int memkind_get_stat(memkind_t kind, memkind_stat_type stat, size_t *value);
 
+///
+/// \brief Print human-readable malloc statistics
+/// \note STANDARD API
+/// \param write_cb pointer to a callback function which prints the statistics,
+///        pass NULL to use the default one
+/// \param cbopaque data passed to write_cb function
+/// \param opts additional options altering the contents of statistics output
+/// \return Memkind operation status, MEMKIND_SUCCESS on success,
+///         MEMKIND_ERROR_INVALID on failure
+///
+int memkind_stats_print(void (*write_cb)(void *, const char *), void *cbopaque,
+                        memkind_stat_print_opt opts);
+
 /* HEAP MANAGEMENT INTERFACE */
 
 ///
@@ -378,7 +568,8 @@ size_t memkind_malloc_usable_size(memkind_t kind, void *ptr);
 
 ///
 /// \brief Allocates memory of the specified kind for an array of num elements
-///        of size bytes each and initializes all bytes in the allocated storage to zero
+///        of size bytes each and initializes all bytes in the allocated storage
+///        to zero
 /// \note STANDARD API
 /// \param kind specified memory kind
 /// \param num number of objects
@@ -388,15 +579,17 @@ size_t memkind_malloc_usable_size(memkind_t kind, void *ptr);
 void *memkind_calloc(memkind_t kind, size_t num, size_t size);
 
 ///
-/// \brief Allocates size bytes of the specified kind and places the address of the allocated memory
-///        in *memptr. The address of the allocated memory will be a multiple of alignment,
-///        which must be a power of two and a multiple of sizeof(void *)
+/// \brief Allocates size bytes of the specified kind and places the address of
+///        the allocated memory in *memptr. The address of the allocated memory
+///        will be a multiple of alignment, which must be a power of two and a
+///        multiple of sizeof(void*)
 /// \note EXPERIMENTAL API
 /// \param kind specified memory kind
 /// \param memptr address of the allocated memory
 /// \param alignment specified alignment of bytes
 /// \param size specified size of bytes
-/// \return Memkind operation status, MEMKIND_SUCCESS on success, EINVAL or ENOMEM on failure
+/// \return Memkind operation status, MEMKIND_SUCCESS on success, EINVAL or
+///         ENOMEM on failure
 ///
 int memkind_posix_memalign(memkind_t kind, void **memptr, size_t alignment,
                            size_t size);
@@ -428,6 +621,26 @@ void memkind_free(memkind_t kind, void *ptr);
 ///
 void *memkind_defrag_reallocate(memkind_t kind, void *ptr);
 
+///
+/// \brief Verifies if file-backed memory kind in the specified directory can be
+///        created with the DAX attribute
+/// \note STANDARD API
+/// \param pmem_dir path to specified directory for PMEM kind
+/// \return Memkind operation status, MEMKIND_SUCCESS on success, other values
+///         on failure
+///
+int memkind_check_dax_path(const char *pmem_dir);
+
+///
+/// \brief Enables/disables background threads
+/// \note STANDARD API
+/// \param state expected state of background threads - true if enabled, false
+///        if disabled
+/// \return Memkind operation status, MEMKIND_SUCCESS on success, other values
+///         on failure
+///
+int memkind_set_bg_threads(bool state);
+
 #ifdef __cplusplus
 }
 #endif
