@@ -2,43 +2,44 @@
 /* Copyright (C) 2014 - 2021 Intel Corporation. */
 
 #define MEMKIND_VERSION_MAJOR 1
-#define MEMKIND_VERSION_MINOR 11
+#define MEMKIND_VERSION_MINOR 12
 #define MEMKIND_VERSION_PATCH 0
 
 #include <memkind.h>
-#include <memkind/internal/memkind_default.h>
-#include <memkind/internal/memkind_hugetlb.h>
+#include <memkind/internal/heap_manager.h>
 #include <memkind/internal/memkind_arena.h>
 #include <memkind/internal/memkind_capacity.h>
-#include <memkind/internal/memkind_local.h>
-#include <memkind/internal/memkind_hbw.h>
-#include <memkind/internal/memkind_regular.h>
-#include <memkind/internal/memkind_gbtlb.h>
 #include <memkind/internal/memkind_dax_kmem.h>
-#include <memkind/internal/memkind_pmem.h>
+#include <memkind/internal/memkind_default.h>
+#include <memkind/internal/memkind_gbtlb.h>
+#include <memkind/internal/memkind_hbw.h>
+#include <memkind/internal/memkind_hugetlb.h>
 #include <memkind/internal/memkind_interleave.h>
-#include <memkind/internal/memkind_private.h>
+#include <memkind/internal/memkind_local.h>
 #include <memkind/internal/memkind_log.h>
+#include <memkind/internal/memkind_pmem.h>
+#include <memkind/internal/memkind_private.h>
+#include <memkind/internal/memkind_regular.h>
 #include <memkind/internal/tbb_wrapper.h>
-#include <memkind/internal/heap_manager.h>
 
 #include "config.h"
 
-#include <numa.h>
-#include <sys/param.h>
-#include <sys/mman.h>
-#include <stdint.h>
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
-#include <limits.h>
 #include <assert.h>
-#include <pthread.h>
 #include <errno.h>
-#include <signal.h>
 #include <fcntl.h>
+#include <limits.h>
+#include <numa.h>
+#include <pthread.h>
+#include <signal.h>
+#include <stdint.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <sys/mman.h>
+#include <sys/param.h>
 #include <unistd.h>
 
+// clang-format off
 #ifdef MEMKIND_ENABLE_HEAP_MANAGER
 #define m_detect_kind(ptr)                      heap_manager_detect_kind(ptr)
 #define m_free(ptr)                             heap_manager_free(ptr)
@@ -54,7 +55,7 @@
 #define m_detect_kind(ptr)                      memkind_arena_detect_kind(ptr)
 #define m_free(ptr)                             memkind_arena_free_with_kind_detect(ptr)
 #define m_realloc(ptr, size)                    memkind_arena_realloc_with_kind_detect(ptr, size)
-#define m_usable_size(ptr)                      memkind_default_malloc_usable_size(NULL, ptr)
+#define m_usable_size(ptr)                      jemk_malloc_usable_size(ptr)
 #define m_defrag_reallocate(ptr)                memkind_arena_defrag_reallocate_with_kind_detect(ptr)
 #define m_get_global_stat(stat, value)          memkind_arena_get_global_stat(stat, value)
 #define m_update_cached_stats                   memkind_arena_update_cached_stats
@@ -62,6 +63,7 @@
 #define m_set_bg_threads(state)                 memkind_arena_set_bg_threads(state)
 #define m_stats_print(write_cb, cbopaque, opts) memkind_arena_stats_print(write_cb, cbopaque, opts)
 #endif
+// clang-format on
 
 /* Clear bits in x, but only this specified in mask. */
 #define CLEAR_BIT(x, mask) ((x) &= (~(mask)))
@@ -71,7 +73,7 @@ extern struct memkind_ops MEMKIND_HBW_PREFERRED_GBTLB_OPS;
 extern struct memkind_ops MEMKIND_GBTLB_OPS;
 
 static struct memkind MEMKIND_DEFAULT_STATIC = {
-    .ops =  &MEMKIND_DEFAULT_OPS,
+    .ops = &MEMKIND_DEFAULT_OPS,
     .partition = MEMKIND_PARTITION_DEFAULT,
     .name = "memkind_default",
     .init_once = PTHREAD_ONCE_INIT,
@@ -254,49 +256,33 @@ static struct memkind MEMKIND_HIGHEST_BANDWIDTH_LOCAL_PREFERRED_STATIC = {
     .init_once = PTHREAD_ONCE_INIT,
 };
 
+// clang-format off
 MEMKIND_EXPORT struct memkind *MEMKIND_DEFAULT = &MEMKIND_DEFAULT_STATIC;
 MEMKIND_EXPORT struct memkind *MEMKIND_HUGETLB = &MEMKIND_HUGETLB_STATIC;
 MEMKIND_EXPORT struct memkind *MEMKIND_INTERLEAVE = &MEMKIND_INTERLEAVE_STATIC;
 MEMKIND_EXPORT struct memkind *MEMKIND_HBW = &MEMKIND_HBW_STATIC;
 MEMKIND_EXPORT struct memkind *MEMKIND_HBW_ALL = &MEMKIND_HBW_ALL_STATIC;
-MEMKIND_EXPORT struct memkind *MEMKIND_HBW_PREFERRED =
-        &MEMKIND_HBW_PREFERRED_STATIC;
-MEMKIND_EXPORT struct memkind *MEMKIND_HBW_HUGETLB =
-        &MEMKIND_HBW_HUGETLB_STATIC;
-MEMKIND_EXPORT struct memkind *MEMKIND_HBW_ALL_HUGETLB =
-        &MEMKIND_HBW_ALL_HUGETLB_STATIC;
-MEMKIND_EXPORT struct memkind *MEMKIND_HBW_PREFERRED_HUGETLB =
-        &MEMKIND_HBW_PREFERRED_HUGETLB_STATIC;
+MEMKIND_EXPORT struct memkind *MEMKIND_HBW_PREFERRED = &MEMKIND_HBW_PREFERRED_STATIC;
+MEMKIND_EXPORT struct memkind *MEMKIND_HBW_HUGETLB = &MEMKIND_HBW_HUGETLB_STATIC;
+MEMKIND_EXPORT struct memkind *MEMKIND_HBW_ALL_HUGETLB = &MEMKIND_HBW_ALL_HUGETLB_STATIC;
+MEMKIND_EXPORT struct memkind *MEMKIND_HBW_PREFERRED_HUGETLB = &MEMKIND_HBW_PREFERRED_HUGETLB_STATIC;
 MEMKIND_EXPORT struct memkind *MEMKIND_HBW_GBTLB = &MEMKIND_HBW_GBTLB_STATIC;
-MEMKIND_EXPORT struct memkind *MEMKIND_HBW_PREFERRED_GBTLB =
-        &MEMKIND_HBW_PREFERRED_GBTLB_STATIC;
+MEMKIND_EXPORT struct memkind *MEMKIND_HBW_PREFERRED_GBTLB = &MEMKIND_HBW_PREFERRED_GBTLB_STATIC;
 MEMKIND_EXPORT struct memkind *MEMKIND_GBTLB = &MEMKIND_GBTLB_STATIC;
-MEMKIND_EXPORT struct memkind *MEMKIND_HBW_INTERLEAVE =
-        &MEMKIND_HBW_INTERLEAVE_STATIC;
+MEMKIND_EXPORT struct memkind *MEMKIND_HBW_INTERLEAVE = &MEMKIND_HBW_INTERLEAVE_STATIC;
 MEMKIND_EXPORT struct memkind *MEMKIND_REGULAR = &MEMKIND_REGULAR_STATIC;
 MEMKIND_EXPORT struct memkind *MEMKIND_DAX_KMEM = &MEMKIND_DAX_KMEM_STATIC;
-MEMKIND_EXPORT struct memkind *MEMKIND_DAX_KMEM_ALL =
-        &MEMKIND_DAX_KMEM_ALL_STATIC;
-MEMKIND_EXPORT struct memkind *MEMKIND_DAX_KMEM_PREFERRED =
-        &MEMKIND_DAX_KMEM_PREFERRED_STATIC;
-MEMKIND_EXPORT struct memkind *MEMKIND_DAX_KMEM_INTERLEAVE =
-        &MEMKIND_DAX_KMEM_INTERLEAVE_STATIC;
-MEMKIND_EXPORT struct memkind *MEMKIND_HIGHEST_CAPACITY =
-        &MEMKIND_HIGHEST_CAPACITY_STATIC;
-MEMKIND_EXPORT struct memkind *MEMKIND_HIGHEST_CAPACITY_PREFERRED =
-        &MEMKIND_HIGHEST_CAPACITY_PREFERRED_STATIC;
-MEMKIND_EXPORT struct memkind *MEMKIND_HIGHEST_CAPACITY_LOCAL =
-        &MEMKIND_HIGHEST_CAPACITY_LOCAL_STATIC;
-MEMKIND_EXPORT struct memkind *MEMKIND_HIGHEST_CAPACITY_LOCAL_PREFERRED =
-        &MEMKIND_HIGHEST_CAPACITY_LOCAL_PREFERRED_STATIC;
-MEMKIND_EXPORT struct memkind *MEMKIND_LOWEST_LATENCY_LOCAL =
-        &MEMKIND_LOWEST_LATENCY_LOCAL_STATIC;
-MEMKIND_EXPORT struct memkind *MEMKIND_LOWEST_LATENCY_LOCAL_PREFERRED =
-        &MEMKIND_LOWEST_LATENCY_LOCAL_PREFERRED_STATIC;
-MEMKIND_EXPORT struct memkind *MEMKIND_HIGHEST_BANDWIDTH_LOCAL =
-        &MEMKIND_HIGHEST_BANDWIDTH_LOCAL_STATIC;
-MEMKIND_EXPORT struct memkind *MEMKIND_HIGHEST_BANDWIDTH_LOCAL_PREFERRED =
-        &MEMKIND_HIGHEST_BANDWIDTH_LOCAL_PREFERRED_STATIC;
+MEMKIND_EXPORT struct memkind *MEMKIND_DAX_KMEM_ALL = &MEMKIND_DAX_KMEM_ALL_STATIC;
+MEMKIND_EXPORT struct memkind *MEMKIND_DAX_KMEM_PREFERRED = &MEMKIND_DAX_KMEM_PREFERRED_STATIC;
+MEMKIND_EXPORT struct memkind *MEMKIND_DAX_KMEM_INTERLEAVE = &MEMKIND_DAX_KMEM_INTERLEAVE_STATIC;
+MEMKIND_EXPORT struct memkind *MEMKIND_HIGHEST_CAPACITY = &MEMKIND_HIGHEST_CAPACITY_STATIC;
+MEMKIND_EXPORT struct memkind *MEMKIND_HIGHEST_CAPACITY_PREFERRED = &MEMKIND_HIGHEST_CAPACITY_PREFERRED_STATIC;
+MEMKIND_EXPORT struct memkind *MEMKIND_HIGHEST_CAPACITY_LOCAL = &MEMKIND_HIGHEST_CAPACITY_LOCAL_STATIC;
+MEMKIND_EXPORT struct memkind *MEMKIND_HIGHEST_CAPACITY_LOCAL_PREFERRED = &MEMKIND_HIGHEST_CAPACITY_LOCAL_PREFERRED_STATIC;
+MEMKIND_EXPORT struct memkind *MEMKIND_LOWEST_LATENCY_LOCAL = &MEMKIND_LOWEST_LATENCY_LOCAL_STATIC;
+MEMKIND_EXPORT struct memkind *MEMKIND_LOWEST_LATENCY_LOCAL_PREFERRED = &MEMKIND_LOWEST_LATENCY_LOCAL_PREFERRED_STATIC;
+MEMKIND_EXPORT struct memkind *MEMKIND_HIGHEST_BANDWIDTH_LOCAL = &MEMKIND_HIGHEST_BANDWIDTH_LOCAL_STATIC;
+MEMKIND_EXPORT struct memkind *MEMKIND_HIGHEST_BANDWIDTH_LOCAL_PREFERRED = &MEMKIND_HIGHEST_BANDWIDTH_LOCAL_PREFERRED_STATIC;
 
 struct memkind_registry {
     struct memkind *partition_map[MEMKIND_MAX_KIND];
@@ -326,7 +312,7 @@ static struct memkind_registry memkind_registry_g = {
         [MEMKIND_PARTITION_DAX_KMEM_INTERLEAVE] = &MEMKIND_DAX_KMEM_INTERLEAVE_STATIC,
         [MEMKIND_PARTITION_HIGHEST_CAPACITY] = &MEMKIND_HIGHEST_CAPACITY_STATIC,
         [MEMKIND_PARTITION_HIGHEST_CAPACITY_PREFERRED] = &MEMKIND_HIGHEST_CAPACITY_PREFERRED_STATIC,
-        [MEMKIND_PARTITION_HIGHEST_CAPACITY_LOCAL] = &MEMKIND_HIGHEST_CAPACITY_LOCAL_STATIC,
+        [MEMKIND_PARTITION_HIGHEST_CAPACITY_LOCAL] =  &MEMKIND_HIGHEST_CAPACITY_LOCAL_STATIC,
         [MEMKIND_PARTITION_HIGHEST_CAPACITY_LOCAL_PREFERRED] = &MEMKIND_HIGHEST_CAPACITY_LOCAL_PREFERRED_STATIC,
         [MEMKIND_PARTITION_LOWEST_LATENCY_LOCAL] = &MEMKIND_LOWEST_LATENCY_LOCAL_STATIC,
         [MEMKIND_PARTITION_LOWEST_LATENCY_LOCAL_PREFERRED] = &MEMKIND_LOWEST_LATENCY_LOCAL_PREFERRED_STATIC,
@@ -336,6 +322,7 @@ static struct memkind_registry memkind_registry_g = {
     MEMKIND_NUM_BASE_KIND,
     PTHREAD_MUTEX_INITIALIZER
 };
+// clang-format on
 
 void *kind_mmap(struct memkind *kind, void *addr, size_t size)
 {
@@ -346,15 +333,16 @@ void *kind_mmap(struct memkind *kind, void *addr, size_t size)
     }
 }
 
-
 static int validate_memtype_bits(memkind_memtype_t memtype)
 {
-    if(memtype == 0) return -1;
+    if (memtype == 0)
+        return -1;
 
     CLEAR_BIT(memtype, MEMKIND_MEMTYPE_DEFAULT);
     CLEAR_BIT(memtype, MEMKIND_MEMTYPE_HIGH_BANDWIDTH);
 
-    if(memtype != 0) return -1;
+    if (memtype != 0)
+        return -1;
     return 0;
 }
 
@@ -362,13 +350,15 @@ static int validate_flags_bits(memkind_bits_t flags)
 {
     CLEAR_BIT(flags, MEMKIND_MASK_PAGE_SIZE_2MB);
 
-    if(flags != 0) return -1;
+    if (flags != 0)
+        return -1;
     return 0;
 }
 
 static int validate_policy(memkind_policy_t policy)
 {
-    if (policy >= MEMKIND_POLICY_MAX_VALUE) return -1;
+    if (policy >= MEMKIND_POLICY_MAX_VALUE)
+        return -1;
     return 0;
 }
 
@@ -379,60 +369,64 @@ struct create_args {
     memkind_memtype_t memtype_flags;
 };
 
+// clang-format off
 static struct create_args supported_args[] = {
 
-    {&MEMKIND_HBW_STATIC,                    MEMKIND_POLICY_BIND_LOCAL,      0,                          MEMKIND_MEMTYPE_HIGH_BANDWIDTH},
-    {&MEMKIND_HBW_HUGETLB_STATIC,            MEMKIND_POLICY_BIND_LOCAL,      MEMKIND_MASK_PAGE_SIZE_2MB, MEMKIND_MEMTYPE_HIGH_BANDWIDTH},
-    {&MEMKIND_HBW_ALL_STATIC,                MEMKIND_POLICY_BIND_ALL,        0,                          MEMKIND_MEMTYPE_HIGH_BANDWIDTH},
-    {&MEMKIND_HBW_ALL_HUGETLB_STATIC,        MEMKIND_POLICY_BIND_ALL,        MEMKIND_MASK_PAGE_SIZE_2MB, MEMKIND_MEMTYPE_HIGH_BANDWIDTH},
-    {&MEMKIND_HBW_PREFERRED_STATIC,          MEMKIND_POLICY_PREFERRED_LOCAL, 0,                          MEMKIND_MEMTYPE_HIGH_BANDWIDTH},
-    {&MEMKIND_HBW_PREFERRED_HUGETLB_STATIC,  MEMKIND_POLICY_PREFERRED_LOCAL, MEMKIND_MASK_PAGE_SIZE_2MB, MEMKIND_MEMTYPE_HIGH_BANDWIDTH},
-    {&MEMKIND_HBW_INTERLEAVE_STATIC,         MEMKIND_POLICY_INTERLEAVE_ALL,  0,                          MEMKIND_MEMTYPE_HIGH_BANDWIDTH},
-    {&MEMKIND_DEFAULT_STATIC,                MEMKIND_POLICY_PREFERRED_LOCAL, 0,                          MEMKIND_MEMTYPE_DEFAULT},
-    {&MEMKIND_HUGETLB_STATIC,                MEMKIND_POLICY_PREFERRED_LOCAL, MEMKIND_MASK_PAGE_SIZE_2MB, MEMKIND_MEMTYPE_DEFAULT},
-    {&MEMKIND_INTERLEAVE_STATIC,             MEMKIND_POLICY_INTERLEAVE_ALL,  0,                          MEMKIND_MEMTYPE_HIGH_BANDWIDTH | MEMKIND_MEMTYPE_DEFAULT},
+    {&MEMKIND_HBW_STATIC, MEMKIND_POLICY_BIND_LOCAL, 0, MEMKIND_MEMTYPE_HIGH_BANDWIDTH},
+    {&MEMKIND_HBW_HUGETLB_STATIC, MEMKIND_POLICY_BIND_LOCAL, MEMKIND_MASK_PAGE_SIZE_2MB, MEMKIND_MEMTYPE_HIGH_BANDWIDTH},
+    {&MEMKIND_HBW_ALL_STATIC, MEMKIND_POLICY_BIND_ALL, 0, MEMKIND_MEMTYPE_HIGH_BANDWIDTH},
+    {&MEMKIND_HBW_ALL_HUGETLB_STATIC, MEMKIND_POLICY_BIND_ALL, MEMKIND_MASK_PAGE_SIZE_2MB, MEMKIND_MEMTYPE_HIGH_BANDWIDTH},
+    {&MEMKIND_HBW_PREFERRED_STATIC, MEMKIND_POLICY_PREFERRED_LOCAL, 0, MEMKIND_MEMTYPE_HIGH_BANDWIDTH},
+    {&MEMKIND_HBW_PREFERRED_HUGETLB_STATIC, MEMKIND_POLICY_PREFERRED_LOCAL, MEMKIND_MASK_PAGE_SIZE_2MB, MEMKIND_MEMTYPE_HIGH_BANDWIDTH},
+    {&MEMKIND_HBW_INTERLEAVE_STATIC, MEMKIND_POLICY_INTERLEAVE_ALL, 0, MEMKIND_MEMTYPE_HIGH_BANDWIDTH},
+    {&MEMKIND_DEFAULT_STATIC, MEMKIND_POLICY_PREFERRED_LOCAL, 0, MEMKIND_MEMTYPE_DEFAULT},
+    {&MEMKIND_HUGETLB_STATIC, MEMKIND_POLICY_PREFERRED_LOCAL, MEMKIND_MASK_PAGE_SIZE_2MB, MEMKIND_MEMTYPE_DEFAULT},
+    {&MEMKIND_INTERLEAVE_STATIC, MEMKIND_POLICY_INTERLEAVE_ALL, 0, MEMKIND_MEMTYPE_HIGH_BANDWIDTH | MEMKIND_MEMTYPE_DEFAULT},
 };
+// clang-format on
 
 /* Kind creation */
 MEMKIND_EXPORT int memkind_create_kind(memkind_memtype_t memtype_flags,
                                        memkind_policy_t policy,
-                                       memkind_bits_t flags,
-                                       memkind_t *kind)
+                                       memkind_bits_t flags, memkind_t *kind)
 {
-    if(validate_memtype_bits(memtype_flags) != 0) {
+    if (validate_memtype_bits(memtype_flags) != 0) {
         log_err("Cannot create kind: incorrect memtype_flags.");
         return MEMKIND_ERROR_INVALID;
     }
 
-    if(validate_flags_bits(flags) != 0) {
+    if (validate_flags_bits(flags) != 0) {
         log_err("Cannot create kind: incorrect flags.");
         return MEMKIND_ERROR_INVALID;
     }
 
-    if(validate_policy(policy) != 0) {
+    if (validate_policy(policy) != 0) {
         log_err("Cannot create kind: incorrect policy.");
         return MEMKIND_ERROR_INVALID;
     }
 
-    if(kind == NULL) {
+    if (kind == NULL) {
         log_err("Cannot create kind: 'kind' is NULL pointer.");
         return MEMKIND_ERROR_INVALID;
     }
 
-    int i, num_supported_args = sizeof(supported_args) / sizeof(struct create_args);
-    for(i = 0; i < num_supported_args; i++) {
-        if((supported_args[i].memtype_flags == memtype_flags) &&
-           (supported_args[i].policy == policy) &&
-           (supported_args[i].flags == flags)) {
+    int i,
+        num_supported_args =
+            sizeof(supported_args) / sizeof(struct create_args);
+    for (i = 0; i < num_supported_args; i++) {
+        if ((supported_args[i].memtype_flags == memtype_flags) &&
+            (supported_args[i].policy == policy) &&
+            (supported_args[i].flags == flags)) {
 
-            if(memkind_check_available(supported_args[i].kind) == 0) {
+            if (memkind_check_available(supported_args[i].kind) == 0) {
                 *kind = supported_args[i].kind;
                 return MEMKIND_SUCCESS;
-            } else if(policy == MEMKIND_POLICY_PREFERRED_LOCAL) {
+            } else if (policy == MEMKIND_POLICY_PREFERRED_LOCAL) {
                 *kind = MEMKIND_DEFAULT;
                 return MEMKIND_SUCCESS;
             }
-            log_err("Cannot create kind: requested memory type is not available.");
+            log_err(
+                "Cannot create kind: requested memory type is not available.");
             return MEMKIND_ERROR_MEMTYPE_NOT_AVAILABLE;
         }
     }
@@ -460,7 +454,8 @@ MEMKIND_EXPORT int memkind_destroy_kind(memkind_t kind)
     int err = kind->ops->destroy(kind);
     for (i = MEMKIND_NUM_BASE_KIND; i < MEMKIND_MAX_KIND; ++i) {
         if (memkind_registry_g.partition_map[i] &&
-            strcmp(kind->name, memkind_registry_g.partition_map[i]->name) == 0) {
+            strcmp(kind->name, memkind_registry_g.partition_map[i]->name) ==
+                0) {
             memkind_destroy_dynamic_kind_from_register(i, kind);
             break;
         }
@@ -476,36 +471,37 @@ MEMKIND_EXPORT memkind_t memkind_detect_kind(void *ptr)
 }
 
 /* Declare weak symbols for allocator decorators */
-extern void memkind_malloc_pre(struct memkind **,
-                               size_t *) __attribute__((weak));
-extern void memkind_malloc_post(struct memkind *, size_t,
-                                void **) __attribute__((weak));
-extern void memkind_calloc_pre(struct memkind **, size_t *,
-                               size_t *) __attribute__((weak));
-extern void memkind_calloc_post(struct memkind *, size_t, size_t,
-                                void **) __attribute__((weak));
+extern void memkind_malloc_pre(struct memkind **, size_t *)
+    __attribute__((weak));
+extern void memkind_malloc_post(struct memkind *, size_t, void **)
+    __attribute__((weak));
+extern void memkind_calloc_pre(struct memkind **, size_t *, size_t *)
+    __attribute__((weak));
+extern void memkind_calloc_post(struct memkind *, size_t, size_t, void **)
+    __attribute__((weak));
 extern void memkind_posix_memalign_pre(struct memkind **, void **, size_t *,
                                        size_t *) __attribute__((weak));
 extern void memkind_posix_memalign_post(struct memkind *, void **, size_t,
                                         size_t, int *) __attribute__((weak));
-extern void memkind_realloc_pre(struct memkind **, void **,
-                                size_t *) __attribute__((weak));
-extern void memkind_realloc_post(struct memkind *, void *, size_t,
-                                 void **) __attribute__((weak));
+extern void memkind_realloc_pre(struct memkind **, void **, size_t *)
+    __attribute__((weak));
+extern void memkind_realloc_post(struct memkind *, void *, size_t, void **)
+    __attribute__((weak));
 extern void memkind_free_pre(struct memkind **, void **) __attribute__((weak));
 extern void memkind_free_post(struct memkind *, void *) __attribute__((weak));
 
 MEMKIND_EXPORT int memkind_get_version()
 {
     return MEMKIND_VERSION_MAJOR * 1000000 + MEMKIND_VERSION_MINOR * 1000 +
-           MEMKIND_VERSION_PATCH;
+        MEMKIND_VERSION_PATCH;
 }
 
 MEMKIND_EXPORT void memkind_error_message(int err, char *msg, size_t size)
 {
     switch (err) {
         case MEMKIND_ERROR_UNAVAILABLE:
-            strncpy(msg, "<memkind> Requested memory kind is not available", size);
+            strncpy(msg, "<memkind> Requested memory kind is not available",
+                    size);
             break;
         case MEMKIND_ERROR_MBIND:
             strncpy(msg, "<memkind> Call to mbind() failed", size);
@@ -517,23 +513,28 @@ MEMKIND_EXPORT void memkind_error_message(int err, char *msg, size_t size)
             strncpy(msg, "<memkind> Call to malloc() failed", size);
             break;
         case MEMKIND_ERROR_ENVIRON:
-            strncpy(msg, "<memkind> Error parsing environment variable (MEMKIND_*)", size);
+            strncpy(msg,
+                    "<memkind> Error parsing environment variable (MEMKIND_*)",
+                    size);
             break;
         case MEMKIND_ERROR_INVALID:
-            strncpy(msg, "<memkind> Invalid input arguments to memkind routine", size);
+            strncpy(msg, "<memkind> Invalid input arguments to memkind routine",
+                    size);
             break;
         case MEMKIND_ERROR_TOOMANY:
-            snprintf(msg, size,
-                     "<memkind> Attempted to initialize more than maximum (%i) number of kinds",
-                     MEMKIND_MAX_KIND);
+            snprintf(
+                msg, size,
+                "<memkind> Attempted to initialize more than maximum (%i) number of kinds",
+                MEMKIND_MAX_KIND);
             break;
         case MEMKIND_ERROR_RUNTIME:
             strncpy(msg, "<memkind> Unspecified run-time error", size);
             break;
         case EINVAL:
-            strncpy(msg,
-                    "<memkind> Alignment must be a power of two and larger than sizeof(void *)",
-                    size);
+            strncpy(
+                msg,
+                "<memkind> Alignment must be a power of two and larger than sizeof(void *)",
+                size);
             break;
         case ENOMEM:
             strncpy(msg, "<memkind> Call to jemk_mallocx() failed", size);
@@ -542,25 +543,28 @@ MEMKIND_EXPORT void memkind_error_message(int err, char *msg, size_t size)
             strncpy(msg, "<memkind> unable to allocate huge pages", size);
             break;
         case MEMKIND_ERROR_BADOPS:
-            strncpy(msg,
-                    "<memkind> memkind_ops structure is poorly formed (missing or incorrect functions)",
-                    size);
+            strncpy(
+                msg,
+                "<memkind> memkind_ops structure is poorly formed (missing or incorrect functions)",
+                size);
             break;
         case MEMKIND_ERROR_MEMTYPE_NOT_AVAILABLE:
-            strncpy(msg, "<memkind> Requested memory type is not available", size);
+            strncpy(msg, "<memkind> Requested memory type is not available",
+                    size);
             break;
         case MEMKIND_ERROR_OPERATION_FAILED:
             strncpy(msg, "<memkind> Operation failed", size);
             break;
         case MEMKIND_ERROR_ARENAS_CREATE:
-            strncpy(msg, "<memkind> Call to jemalloc's arenas.create () failed", size);
+            strncpy(msg, "<memkind> Call to jemalloc's arenas.create () failed",
+                    size);
             break;
         default:
             snprintf(msg, size, "<memkind> Undefined error number: %i", err);
             break;
     }
     if (size > 0) {
-        msg[size-1] = '\0';
+        msg[size - 1] = '\0';
     }
 }
 
@@ -571,7 +575,8 @@ void memkind_init(memkind_t kind, bool check_numa)
     if (check_numa) {
         int err = numa_available();
         if (err) {
-            log_fatal("[%s] NUMA not available (error code:%d).", kind->name, err);
+            log_fatal("[%s] NUMA not available (error code:%d).", kind->name,
+                      err);
             abort();
         }
     }
@@ -586,7 +591,8 @@ char *memkind_get_env(const char *name)
 #endif
 }
 
-static void nop(void) {}
+static void nop(void)
+{}
 
 static int memkind_create(struct memkind_ops *ops, const char *name,
                           struct memkind **kind)
@@ -600,18 +606,15 @@ static int memkind_create(struct memkind_ops *ops, const char *name,
         assert(0 && "failed to acquire mutex");
 
     if (memkind_registry_g.num_kind == MEMKIND_MAX_KIND) {
-        log_err("Attempted to initialize more than maximum (%i) number of kinds.",
-                MEMKIND_MAX_KIND);
+        log_err(
+            "Attempted to initialize more than maximum (%i) number of kinds.",
+            MEMKIND_MAX_KIND);
         err = MEMKIND_ERROR_TOOMANY;
         goto exit;
     }
-    if (ops->create == NULL ||
-        ops->destroy == NULL ||
-        ops->malloc == NULL ||
-        ops->calloc == NULL ||
-        ops->realloc == NULL ||
-        ops->posix_memalign == NULL ||
-        ops->free == NULL ||
+    if (ops->create == NULL || ops->destroy == NULL || ops->malloc == NULL ||
+        ops->calloc == NULL || ops->realloc == NULL ||
+        ops->posix_memalign == NULL || ops->free == NULL ||
         ops->init_once != NULL) {
         err = MEMKIND_ERROR_BADOPS;
         goto exit;
@@ -620,7 +623,8 @@ static int memkind_create(struct memkind_ops *ops, const char *name,
         if (memkind_registry_g.partition_map[i] == NULL) {
             id_kind = i;
             break;
-        } else if (strcmp(name, memkind_registry_g.partition_map[i]->name) == 0) {
+        } else if (strcmp(name, memkind_registry_g.partition_map[i]->name) ==
+                   0) {
             log_err("Kind with the name %s already exists", name);
             err = MEMKIND_ERROR_INVALID;
             goto exit;
@@ -644,7 +648,7 @@ static int memkind_create(struct memkind_ops *ops, const char *name,
 
     (*kind)->init_once = PTHREAD_ONCE_INIT;
     pthread_once(&(*kind)->init_once,
-                 nop); //this is done to avoid init_once for dynamic kinds
+                 nop); // this is done to avoid init_once for dynamic kinds
 exit:
     if (pthread_mutex_unlock(&memkind_registry_g.lock) != 0)
         assert(0 && "failed to release mutex");
@@ -667,7 +671,8 @@ static int memkind_use_other_heap_manager(void)
 #ifdef __GNUC__
 __attribute__((constructor))
 #endif
-static void memkind_construct(void)
+static void
+memkind_construct(void)
 {
     if (!memkind_use_other_heap_manager()) {
         const char *env = memkind_get_env("MEMKIND_BACKGROUND_THREAD_LIMIT");
@@ -676,18 +681,22 @@ static void memkind_construct(void)
             errno = 0;
             size_t thread_limit = strtoul(env, &end, 10);
             if (thread_limit > UINT_MAX || *end != '\0' || errno != 0) {
-                log_fatal("Error: Wrong value of MEMKIND_BACKGROUND_THREAD_LIMIT=%s", env);
+                log_fatal(
+                    "Error: Wrong value of MEMKIND_BACKGROUND_THREAD_LIMIT=%s",
+                    env);
                 abort();
             }
             memkind_arena_set_max_bg_threads(thread_limit);
         }
     }
+    memkind_set_hog_memory(memkind_get_env("MEMKIND_HOG_MEMORY"));
 }
 
 #ifdef __GNUC__
 __attribute__((destructor))
 #endif
-static int memkind_finalize(void)
+static int
+memkind_finalize(void)
 {
     struct memkind *kind;
     unsigned i;
@@ -737,17 +746,13 @@ MEMKIND_EXPORT size_t memkind_malloc_usable_size(struct memkind *kind,
 
 MEMKIND_EXPORT void *memkind_malloc(struct memkind *kind, size_t size)
 {
-    void *result;
-
-    pthread_once(&kind->init_once, kind->ops->init_once);
-
 #ifdef MEMKIND_DECORATION_ENABLED
     if (memkind_malloc_pre) {
         memkind_malloc_pre(&kind, &size);
     }
 #endif
 
-    result = kind->ops->malloc(kind, size);
+    void *result = kind->ops->malloc(kind, size);
 
 #ifdef MEMKIND_DECORATION_ENABLED
     if (memkind_malloc_post) {
@@ -761,17 +766,13 @@ MEMKIND_EXPORT void *memkind_malloc(struct memkind *kind, size_t size)
 MEMKIND_EXPORT void *memkind_calloc(struct memkind *kind, size_t num,
                                     size_t size)
 {
-    void *result;
-
-    pthread_once(&kind->init_once, kind->ops->init_once);
-
 #ifdef MEMKIND_DECORATION_ENABLED
     if (memkind_calloc_pre) {
         memkind_calloc_pre(&kind, &num, &size);
     }
 #endif
 
-    result = kind->ops->calloc(kind, num, size);
+    void *result = kind->ops->calloc(kind, num, size);
 
 #ifdef MEMKIND_DECORATION_ENABLED
     if (memkind_calloc_post) {
@@ -783,20 +784,15 @@ MEMKIND_EXPORT void *memkind_calloc(struct memkind *kind, size_t num,
 }
 
 MEMKIND_EXPORT int memkind_posix_memalign(struct memkind *kind, void **memptr,
-                                          size_t alignment,
-                                          size_t size)
+                                          size_t alignment, size_t size)
 {
-    int err;
-
-    pthread_once(&kind->init_once, kind->ops->init_once);
-
 #ifdef MEMKIND_DECORATION_ENABLED
     if (memkind_posix_memalign_pre) {
         memkind_posix_memalign_pre(&kind, memptr, &alignment, &size);
     }
 #endif
 
-    err = kind->ops->posix_memalign(kind, memptr, alignment, size);
+    int err = kind->ops->posix_memalign(kind, memptr, alignment, size);
 
 #ifdef MEMKIND_DECORATION_ENABLED
     if (memkind_posix_memalign_post) {
@@ -821,7 +817,6 @@ MEMKIND_EXPORT void *memkind_realloc(struct memkind *kind, void *ptr,
     if (!kind) {
         result = m_realloc(ptr, size);
     } else {
-        pthread_once(&kind->init_once, kind->ops->init_once);
         result = kind->ops->realloc(kind, ptr, size);
     }
 
@@ -844,7 +839,6 @@ MEMKIND_EXPORT void memkind_free(struct memkind *kind, void *ptr)
     if (!kind) {
         m_free(ptr);
     } else {
-        pthread_once(&kind->init_once, kind->ops->init_once);
         kind->ops->free(kind, ptr);
     }
 
@@ -857,8 +851,8 @@ MEMKIND_EXPORT void memkind_free(struct memkind *kind, void *ptr)
 
 MEMKIND_EXPORT struct memkind_config *memkind_config_new(void)
 {
-    struct memkind_config *cfg = (struct memkind_config *)malloc(sizeof(
-                                                                     struct memkind_config));
+    struct memkind_config *cfg =
+        (struct memkind_config *)malloc(sizeof(struct memkind_config));
     return cfg;
 }
 
@@ -879,8 +873,9 @@ MEMKIND_EXPORT void memkind_config_set_size(struct memkind_config *cfg,
     cfg->pmem_size = pmem_size;
 }
 
-MEMKIND_EXPORT void memkind_config_set_memory_usage_policy(
-    struct memkind_config *cfg, memkind_mem_usage_policy policy)
+MEMKIND_EXPORT void
+memkind_config_set_memory_usage_policy(struct memkind_config *cfg,
+                                       memkind_mem_usage_policy policy)
 {
     cfg->policy = policy;
 }
@@ -891,7 +886,9 @@ MEMKIND_EXPORT int memkind_create_pmem(const char *dir, size_t max_size,
     int oerrno;
 
     if (max_size && max_size < MEMKIND_PMEM_MIN_SIZE) {
-        log_err("Cannot create pmem: invalid size.");
+        log_err(
+            "Cannot create pmem: invalid size: %zu - size must be equal 0 or greater than or equal to MEMKIND_PMEM_MIN_SIZE",
+            max_size);
         return MEMKIND_ERROR_INVALID;
     }
 
@@ -908,7 +905,7 @@ MEMKIND_EXPORT int memkind_create_pmem(const char *dir, size_t max_size,
         goto exit;
     }
 
-    snprintf(name, sizeof (name), "pmem%08x", fd);
+    snprintf(name, sizeof(name), "pmem%08x", fd);
 
     err = memkind_create(&MEMKIND_PMEM_OPS, name, kind);
     if (err) {
@@ -921,7 +918,7 @@ MEMKIND_EXPORT int memkind_create_pmem(const char *dir, size_t max_size,
     priv->offset = 0;
     priv->current_size = 0;
     priv->max_size = max_size;
-    priv->dir = jemk_malloc(strlen(dir)+1);
+    priv->dir = jemk_malloc(strlen(dir) + 1);
     if (!priv->dir) {
         goto exit;
     }
@@ -932,7 +929,7 @@ MEMKIND_EXPORT int memkind_create_pmem(const char *dir, size_t max_size,
 exit:
     oerrno = errno;
     if (fd != -1) {
-        (void) close(fd);
+        (void)close(fd);
     }
     errno = oerrno;
     return err;
@@ -949,14 +946,12 @@ MEMKIND_EXPORT int memkind_create_pmem_with_config(struct memkind_config *cfg,
     return status;
 }
 
-
 static int memkind_get_kind_by_partition_internal(int partition,
                                                   struct memkind **kind)
 {
     int err = MEMKIND_SUCCESS;
 
-    if (MEMKIND_LIKELY(partition >= 0 &&
-                       partition < MEMKIND_MAX_KIND &&
+    if (MEMKIND_LIKELY(partition >= 0 && partition < MEMKIND_MAX_KIND &&
                        memkind_registry_g.partition_map[partition] != NULL)) {
         *kind = memkind_registry_g.partition_map[partition];
     } else {
@@ -1011,8 +1006,9 @@ MEMKIND_EXPORT int memkind_set_bg_threads(bool state)
     return m_set_bg_threads(state);
 }
 
-MEMKIND_EXPORT int memkind_stats_print(void (*write_cb) (void *, const char *),
-                                       void *cbopaque, memkind_stat_print_opt opts)
+MEMKIND_EXPORT int memkind_stats_print(void (*write_cb)(void *, const char *),
+                                       void *cbopaque,
+                                       memkind_stat_print_opt opts)
 {
     return m_stats_print(write_cb, cbopaque, opts);
 }
