@@ -1,60 +1,38 @@
-/*
- * Copyright (C) 2014 - 2019 Intel Corporation.
- * All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions are met:
- * 1. Redistributions of source code must retain the above copyright notice(s),
- *    this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright notice(s),
- *    this list of conditions and the following disclaimer in the documentation
- *    and/or other materials provided with the distribution.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER(S) ``AS IS'' AND ANY EXPRESS
- * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
- * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO
- * EVENT SHALL THE COPYRIGHT HOLDER(S) BE LIABLE FOR ANY DIRECT, INDIRECT,
- * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
- * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
- * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
- * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
- * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- */
+// SPDX-License-Identifier: BSD-2-Clause
+/* Copyright (C) 2014 - 2021 Intel Corporation. */
 
 #include <memkind.h>
-#include <memkind/internal/memkind_default.h>
 #include <memkind/internal/memkind_arena.h>
-#include <memkind/internal/memkind_private.h>
+#include <memkind/internal/memkind_default.h>
 #include <memkind/internal/memkind_log.h>
+#include <memkind/internal/memkind_private.h>
 
-#include <stdlib.h>
-#include <stdio.h>
-#include <limits.h>
-#include <stdint.h>
-#include <unistd.h>
-#include <pthread.h>
+#include <assert.h>
 #include <errno.h>
+#include <jemalloc/jemalloc.h>
+#include <limits.h>
 #include <numa.h>
 #include <numaif.h>
-#include <jemalloc/jemalloc.h>
-#include <utmpx.h>
+#include <pthread.h>
 #include <sched.h>
-#include <limits.h>
+#include <stdint.h>
+#include <stdio.h>
+#include <stdlib.h>
 #include <sys/mman.h>
 #include <sys/param.h>
-#include <assert.h>
+#include <unistd.h>
+#include <utmpx.h>
 
 #include "config.h"
 
-#define HUGE_PAGE_SIZE (1ull << MEMKIND_MASK_PAGE_SIZE_2MB)
+#define HUGE_PAGE_SIZE  (1ull << MEMKIND_MASK_PAGE_SIZE_2MB)
 #define PAGE_2_BYTES(x) ((x) << 12)
 
+/* Clear bits in x, but only this specified in mask. */
+#define CLEAR_BIT(x, mask) ((x) &= (~(mask)))
+
 static const char *const global_stats[MEMKIND_STAT_TYPE_MAX_VALUE] = {
-    "stats.resident",
-    "stats.active",
-    "stats.allocated"
-};
+    "stats.resident", "stats.active", "stats.allocated"};
 
 #define ARENA_STAT_MAX 2
 
@@ -63,14 +41,15 @@ struct stats_arena {
     unsigned stats_no;
 };
 
-static const struct stats_arena arena_stats [MEMKIND_STAT_TYPE_MAX_VALUE] = {
-    { .stats = {"stats.arenas.%u.resident", NULL}, .stats_no = 1},
-    { .stats = {"stats.arenas.%u.pactive", NULL}, .stats_no = 1},
-    { .stats = {"stats.arenas.%u.small.allocated", "stats.arenas.%u.large.allocated"}, .stats_no = 2},
+static const struct stats_arena arena_stats[MEMKIND_STAT_TYPE_MAX_VALUE] = {
+    {.stats = {"stats.arenas.%u.resident", NULL}, .stats_no = 1},
+    {.stats = {"stats.arenas.%u.pactive", NULL}, .stats_no = 1},
+    {.stats = {"stats.arenas.%u.small.allocated",
+               "stats.arenas.%u.large.allocated"},
+     .stats_no = 2},
 };
 
 static void *jemk_mallocx_check(size_t size, int flags);
-static void *jemk_rallocx_check(void *ptr, size_t size, int flags);
 static void tcache_finalize(void *args);
 
 static unsigned integer_log2(unsigned v)
@@ -93,14 +72,16 @@ MEMKIND_EXPORT int memkind_set_arena_map_len(struct memkind *kind)
     if (kind->ops->get_arena == memkind_bijective_get_arena) {
         kind->arena_map_len = 1;
     } else if (kind->ops->get_arena == memkind_thread_get_arena) {
-        char *arena_num_env = secure_getenv("MEMKIND_ARENA_NUM_PER_KIND");
+        char *arena_num_env = memkind_get_env("MEMKIND_ARENA_NUM_PER_KIND");
 
         if (arena_num_env) {
-            unsigned long int arena_num_value = strtoul(arena_num_env, NULL, 10);
+            unsigned long int arena_num_value =
+                strtoul(arena_num_env, NULL, 10);
 
             if ((arena_num_value == 0) || (arena_num_value > INT_MAX)) {
-                log_err("Wrong MEMKIND_ARENA_NUM_PER_KIND environment value: %lu.",
-                        arena_num_value);
+                log_err(
+                    "Wrong MEMKIND_ARENA_NUM_PER_KIND environment value: %lu.",
+                    arena_num_value);
                 return MEMKIND_ERROR_ENVIRON;
             }
 
@@ -109,7 +90,8 @@ MEMKIND_EXPORT int memkind_set_arena_map_len(struct memkind *kind)
             int calculated_arena_num = numa_num_configured_cpus() * 4;
 
 #if ARENA_LIMIT_PER_KIND != 0
-            calculated_arena_num = MIN((ARENA_LIMIT_PER_KIND), calculated_arena_num);
+            calculated_arena_num =
+                MIN((ARENA_LIMIT_PER_KIND), calculated_arena_num);
 #endif
             kind->arena_map_len = calculated_arena_num;
         }
@@ -127,17 +109,27 @@ static int arena_init_status;
 static pthread_key_t tcache_key;
 static bool memkind_hog_memory;
 
-static void arena_config_init()
+bool memkind_get_hog_memory(void)
 {
-    const char *str = secure_getenv("MEMKIND_HOG_MEMORY");
-    memkind_hog_memory = str && str[0] == '1';
+    return memkind_hog_memory;
+}
 
+void memkind_set_hog_memory(const char *str)
+{
+    memkind_hog_memory = str && str[0] == '1';
+}
+
+static void arena_config_init(void)
+{
     arena_init_status = pthread_key_create(&tcache_key, tcache_finalize);
 }
 
-#define MALLOCX_ARENA_MAX           0xffe        // copy-pasted from jemalloc/internal/jemalloc_internal.h
-#define DIRTY_DECAY_MS_DEFAULT      10000        // copy-pasted from jemalloc/internal/arena_types.h DIRTY_DECAY_MS_DEFAULT
-#define DIRTY_DECAY_MS_CONSERVATIVE     0
+#define MALLOCX_ARENA_MAX                                                      \
+    0xffe // copy-pasted from jemalloc/internal/jemalloc_internal.h
+#define DIRTY_DECAY_MS_DEFAULT                                                 \
+    10000 // copy-pasted from jemalloc/internal/arena_types.h
+          // DIRTY_DECAY_MS_DEFAULT
+#define DIRTY_DECAY_MS_CONSERVATIVE 0
 
 static struct memkind *arena_registry_g[MALLOCX_ARENA_MAX];
 static pthread_mutex_t arena_registry_write_lock;
@@ -158,9 +150,9 @@ static void *alloc_aligned_slow(size_t size, size_t alignment,
     size_t extended_size = size + alignment;
     void *ptr;
 
-    ptr = kind_mmap(kind,  NULL, extended_size);
+    ptr = kind_mmap(kind, NULL, extended_size);
 
-    if(ptr == MAP_FAILED) {
+    if (ptr == MAP_FAILED) {
         return NULL;
     }
 
@@ -181,14 +173,9 @@ static void *alloc_aligned_slow(size_t size, size_t alignment,
     return (void *)aligned_addr;
 }
 
-
-void *arena_extent_alloc(extent_hooks_t *extent_hooks,
-                         void *new_addr,
-                         size_t size,
-                         size_t alignment,
-                         bool *zero,
-                         bool *commit,
-                         unsigned arena_ind)
+void *arena_extent_alloc(extent_hooks_t *extent_hooks, void *new_addr,
+                         size_t size, size_t alignment, bool *zero,
+                         bool *commit, unsigned arena_ind)
 {
     struct memkind *kind = get_kind_by_arena(arena_ind);
 
@@ -208,10 +195,10 @@ void *arena_extent_alloc(extent_hooks_t *extent_hooks,
         return NULL;
     }
 
-    if ((uintptr_t)addr & (alignment-1)) {
+    if ((uintptr_t)addr & (alignment - 1)) {
         munmap(addr, size);
         addr = alloc_aligned_slow(size, alignment, kind);
-        if(addr == NULL) {
+        if (addr == NULL) {
             return NULL;
         }
     }
@@ -222,87 +209,65 @@ void *arena_extent_alloc(extent_hooks_t *extent_hooks,
     return addr;
 }
 
-void *arena_extent_alloc_hugetlb(extent_hooks_t *extent_hooks,
-                                 void *new_addr,
-                                 size_t size,
-                                 size_t alignment,
-                                 bool *zero,
-                                 bool *commit,
-                                 unsigned arena_ind)
+void *arena_extent_alloc_hugetlb(extent_hooks_t *extent_hooks, void *new_addr,
+                                 size_t size, size_t alignment, bool *zero,
+                                 bool *commit, unsigned arena_ind)
 {
-    //round up to huge page size
+    // round up to huge page size
     size = (size + (HUGE_PAGE_SIZE - 1)) & ~(HUGE_PAGE_SIZE - 1);
-    return arena_extent_alloc(extent_hooks, new_addr, size, alignment, zero, commit,
-                              arena_ind);
+    return arena_extent_alloc(extent_hooks, new_addr, size, alignment, zero,
+                              commit, arena_ind);
 }
 
-bool arena_extent_dalloc(extent_hooks_t *extent_hooks,
-                         void *addr,
-                         size_t size,
-                         bool committed,
-                         unsigned arena_ind)
+bool arena_extent_dalloc(extent_hooks_t *extent_hooks, void *addr, size_t size,
+                         bool committed, unsigned arena_ind)
 {
     return true;
 }
 
-bool arena_extent_commit(extent_hooks_t *extent_hooks,
-                         void *addr,
-                         size_t size,
-                         size_t offset,
-                         size_t length,
-                         unsigned arena_ind)
+bool arena_extent_commit(extent_hooks_t *extent_hooks, void *addr, size_t size,
+                         size_t offset, size_t length, unsigned arena_ind)
 {
     return false;
 }
 
-bool arena_extent_decommit(extent_hooks_t *extent_hooks,
-                           void *addr,
-                           size_t size,
-                           size_t offset,
-                           size_t length,
+bool arena_extent_decommit(extent_hooks_t *extent_hooks, void *addr,
+                           size_t size, size_t offset, size_t length,
                            unsigned arena_ind)
 {
     return true;
 }
 
-bool arena_extent_purge(extent_hooks_t *extent_hooks,
-                        void *addr,
-                        size_t size,
-                        size_t offset,
-                        size_t length,
-                        unsigned arena_ind)
+bool arena_extent_purge_hog_memory(extent_hooks_t *extent_hooks, void *addr,
+                                   size_t size, size_t offset, size_t length,
+                                   unsigned arena_ind)
 {
-    if (memkind_hog_memory) {
-        return true;
-    }
+    return true;
+}
 
+bool arena_extent_purge(extent_hooks_t *extent_hooks, void *addr, size_t size,
+                        size_t offset, size_t length, unsigned arena_ind)
+{
     int err = madvise(addr + offset, length, MADV_DONTNEED);
     return (err != 0);
 }
 
-bool arena_extent_split(extent_hooks_t *extent_hooks,
-                        void *addr,
-                        size_t size,
-                        size_t size_a,
-                        size_t size_b,
-                        bool committed,
+bool arena_extent_split(extent_hooks_t *extent_hooks, void *addr, size_t size,
+                        size_t size_a, size_t size_b, bool committed,
                         unsigned arena_ind)
 {
     return false;
 }
 
-bool arena_extent_merge(extent_hooks_t *extent_hooks,
-                        void *addr_a,
-                        size_t size_a,
-                        void *addr_b,
-                        size_t size_b,
-                        bool committed,
-                        unsigned arena_ind)
+bool arena_extent_merge(extent_hooks_t *extent_hooks, void *addr_a,
+                        size_t size_a, void *addr_b, size_t size_b,
+                        bool committed, unsigned arena_ind)
 {
     return false;
 }
 
-extent_hooks_t arena_extent_hooks = {
+// clang-format off
+static extent_hooks_t arena_extent_hooks = {
     .alloc = arena_extent_alloc,
     .dalloc = arena_extent_dalloc,
     .commit = arena_extent_commit,
@@ -312,7 +277,17 @@ extent_hooks_t arena_extent_hooks = {
     .merge = arena_extent_merge
 };
 
-extent_hooks_t arena_extent_hooks_hugetlb = {
+static extent_hooks_t arena_extent_hooks_hog_memory = {
+    .alloc = arena_extent_alloc,
+    .dalloc = arena_extent_dalloc,
+    .commit = arena_extent_commit,
+    .decommit = arena_extent_decommit,
+    .purge_lazy = arena_extent_purge_hog_memory,
+    .split = arena_extent_split,
+    .merge = arena_extent_merge
+};
+
+static extent_hooks_t arena_extent_hooks_hugetlb = {
     .alloc = arena_extent_alloc_hugetlb,
     .dalloc = arena_extent_dalloc,
     .commit = arena_extent_commit,
@@ -322,16 +297,31 @@ extent_hooks_t arena_extent_hooks_hugetlb = {
     .merge = arena_extent_merge
 };
 
+static extent_hooks_t arena_extent_hooks_hugetlb_hog_memory = {
+    .alloc = arena_extent_alloc_hugetlb,
+    .dalloc = arena_extent_dalloc,
+    .commit = arena_extent_commit,
+    .decommit = arena_extent_decommit,
+    .purge_lazy = arena_extent_purge_hog_memory,
+    .split = arena_extent_split,
+    .merge = arena_extent_merge
+};
+// clang-format on
+
 extent_hooks_t *get_extent_hooks_by_kind(struct memkind *kind)
 {
-    if (kind == MEMKIND_HUGETLB
-        || kind == MEMKIND_HBW_HUGETLB
-        || kind == MEMKIND_HBW_ALL_HUGETLB
-        || kind == MEMKIND_HBW_PREFERRED_HUGETLB) {
+    if (kind == MEMKIND_HUGETLB || kind == MEMKIND_HBW_HUGETLB ||
+        kind == MEMKIND_HBW_ALL_HUGETLB ||
+        kind == MEMKIND_HBW_PREFERRED_HUGETLB) {
+        if (memkind_get_hog_memory()) {
+            return &arena_extent_hooks_hugetlb_hog_memory;
+        }
         return &arena_extent_hooks_hugetlb;
-    } else {
-        return &arena_extent_hooks;
     }
+    if (memkind_get_hog_memory()) {
+        return &arena_extent_hooks_hog_memory;
+    }
+    return &arena_extent_hooks;
 }
 
 MEMKIND_EXPORT int memkind_arena_create_map(struct memkind *kind,
@@ -340,12 +330,12 @@ MEMKIND_EXPORT int memkind_arena_create_map(struct memkind *kind,
     int err;
 
     pthread_once(&arena_config_once, arena_config_init);
-    if(arena_init_status) {
+    if (arena_init_status) {
         return arena_init_status;
     }
 
     err = memkind_set_arena_map_len(kind);
-    if(err) {
+    if (err) {
         return err;
     }
 #ifdef MEMKIND_TLS
@@ -359,25 +349,27 @@ MEMKIND_EXPORT int memkind_arena_create_map(struct memkind *kind,
     unsigned i;
     size_t unsigned_size = sizeof(unsigned int);
     kind->arena_zero = UINT_MAX;
-    for(i = 0; i<kind->arena_map_len; i++) {
+    for (i = 0; i < kind->arena_map_len; i++) {
         unsigned arena_index;
-        //create new arena with consecutive index
-        err = jemk_mallctl("arenas.create", (void *)&arena_index, &unsigned_size, NULL,
-                           0);
-        if(err) {
+        // create new arena with consecutive index
+        err = jemk_mallctl("arenas.create", (void *)&arena_index,
+                           &unsigned_size, NULL, 0);
+        if (err) {
             log_err("Could not create arena.");
             err = MEMKIND_ERROR_ARENAS_CREATE;
             goto exit;
         }
-        //store arena with lowest index (arenas could be created in descending/ascending order)
-        if(kind->arena_zero > arena_index) {
+        // store arena with lowest index (arenas could be created in
+        // descending/ascending order)
+        if (kind->arena_zero > arena_index) {
             kind->arena_zero = arena_index;
         }
-        //setup extent_hooks for newly created arena
+        // setup extent_hooks for newly created arena
         char cmd[64];
         snprintf(cmd, sizeof(cmd), "arena.%u.extent_hooks", arena_index);
-        err = jemk_mallctl(cmd, NULL, NULL, (void *)&hooks, sizeof(extent_hooks_t *));
-        if(err) {
+        err = jemk_mallctl(cmd, NULL, NULL, (void *)&hooks,
+                           sizeof(extent_hooks_t *));
+        if (err) {
             goto exit;
         }
         arena_registry_g[arena_index] = kind;
@@ -390,7 +382,8 @@ exit:
 }
 
 MEMKIND_EXPORT int memkind_arena_create(struct memkind *kind,
-                                        struct memkind_ops *ops, const char *name)
+                                        struct memkind_ops *ops,
+                                        const char *name)
 {
     int err = memkind_default_create(kind, ops, name);
     if (!err) {
@@ -424,7 +417,6 @@ MEMKIND_EXPORT int memkind_arena_destroy(struct memkind *kind)
 #endif
     }
 
-    memkind_default_destroy(kind);
     return 0;
 }
 
@@ -434,49 +426,29 @@ int memkind_arena_finalize(struct memkind *kind)
 }
 
 // max allocation size to be cached by tcache mechanism
-#define TCACHE_MAX (1<<(JEMALLOC_TCACHE_CLASS))
+#define TCACHE_MAX (1 << (JEMALLOC_TCACHE_CLASS))
 
 static void tcache_finalize(void *args)
 {
     int i;
     unsigned *tcache_map = args;
-    for(i = 0; i<MEMKIND_NUM_BASE_KIND; i++) {
-        if(tcache_map[i] != 0) {
+    for (i = 0; i < MEMKIND_NUM_BASE_KIND; i++) {
+        if (tcache_map[i] != 0) {
             jemk_mallctl("tcache.destroy", NULL, NULL, (void *)&tcache_map[i],
                          sizeof(unsigned));
         }
     }
 }
 
-static inline int memkind_lookup_arena(void *ptr, unsigned int *arena)
-{
-    size_t sz = sizeof(unsigned);
-    unsigned temp_arena;
-    int err = jemk_mallctl("arenas.lookup", &temp_arena, &sz, &ptr, sizeof(ptr));
-
-    if (err) {
-        log_err("Could not found arena, err=%d", err);
-        return 1;
-    }
-
-    *arena = temp_arena;
-    return 0;
-}
-
 MEMKIND_EXPORT struct memkind *memkind_arena_detect_kind(void *ptr)
 {
     if (!ptr) {
         return NULL;
     }
-    struct memkind *kind = NULL;
-    unsigned arena;
-    int err = memkind_lookup_arena(ptr, &arena);
-    if (MEMKIND_LIKELY(!err)) {
-        kind = get_kind_by_arena(arena);
-    }
 
-    /* if no kind was associated with arena it means that allocation doesn't come from
-       jemk_*allocx API - it is jemk_*alloc API (MEMKIND_DEFAULT) */
+    struct memkind *kind = get_kind_by_arena((unsigned)jemk_arenalookupx(ptr));
+    /* if no kind was associated with arena it means that allocation doesn't
+       come from jemk_*allocx API - it is jemk_*alloc API (MEMKIND_DEFAULT) */
 
     return (kind) ? kind : MEMKIND_DEFAULT;
 }
@@ -484,25 +456,26 @@ MEMKIND_EXPORT struct memkind *memkind_arena_detect_kind(void *ptr)
 static inline int get_tcache_flag(unsigned partition, size_t size)
 {
 
-    // do not cache allocation larger than tcache_max nor those coming from non-static kinds
-    if(size > TCACHE_MAX || partition >= MEMKIND_NUM_BASE_KIND) {
+    // do not cache allocation larger than tcache_max nor those coming from
+    // non-static kinds
+    if (size > TCACHE_MAX || partition >= MEMKIND_NUM_BASE_KIND) {
         return MALLOCX_TCACHE_NONE;
     }
 
     unsigned *tcache_map = pthread_getspecific(tcache_key);
-    if(tcache_map == NULL) {
-        tcache_map = calloc(MEMKIND_NUM_BASE_KIND, sizeof(unsigned));
-        if(tcache_map == NULL) {
+    if (tcache_map == NULL) {
+        tcache_map = jemk_calloc(MEMKIND_NUM_BASE_KIND, sizeof(unsigned));
+        if (tcache_map == NULL) {
             return MALLOCX_TCACHE_NONE;
         }
         pthread_setspecific(tcache_key, (void *)tcache_map);
     }
 
-    if(MEMKIND_UNLIKELY(tcache_map[partition] == 0)) {
+    if (MEMKIND_UNLIKELY(tcache_map[partition] == 0)) {
         size_t unsigned_size = sizeof(unsigned);
         int err = jemk_mallctl("tcache.create", (void *)&tcache_map[partition],
                                &unsigned_size, NULL, 0);
-        if(err) {
+        if (err) {
             log_err("Could not acquire tcache, err=%d", err);
             return MALLOCX_TCACHE_NONE;
         }
@@ -512,30 +485,30 @@ static inline int get_tcache_flag(unsigned partition, size_t size)
 
 MEMKIND_EXPORT void *memkind_arena_malloc(struct memkind *kind, size_t size)
 {
-    void *result = NULL;
+    pthread_once(&kind->init_once, kind->ops->init_once);
     unsigned arena;
 
     int err = kind->ops->get_arena(kind, &arena, size);
     if (MEMKIND_LIKELY(!err)) {
-        result = jemk_mallocx_check(size,
-                                    MALLOCX_ARENA(arena) | get_tcache_flag(kind->partition, size));
+        return jemk_mallocx_check(size,
+                                  MALLOCX_ARENA(arena) |
+                                      get_tcache_flag(kind->partition, size));
     }
-    return result;
+    return NULL;
 }
 
 static void *memkind_arena_malloc_no_tcache(struct memkind *kind, size_t size)
 {
-    void *result = NULL;
     if (kind == MEMKIND_DEFAULT) {
-        result = jemk_mallocx_check(size, MALLOCX_TCACHE_NONE);
-    } else {
-        unsigned arena;
-        int err = kind->ops->get_arena(kind, &arena, size);
-        if (MEMKIND_LIKELY(!err)) {
-            result = jemk_mallocx_check(size, MALLOCX_ARENA(arena) | MALLOCX_TCACHE_NONE);
-        }
+        return jemk_mallocx_check(size, MALLOCX_TCACHE_NONE);
     }
-    return result;
+    unsigned arena;
+    int err = kind->ops->get_arena(kind, &arena, size);
+    if (MEMKIND_LIKELY(!err)) {
+        return jemk_mallocx_check(size,
+                                  MALLOCX_ARENA(arena) | MALLOCX_TCACHE_NONE);
+    }
+    return NULL;
 }
 
 MEMKIND_EXPORT void memkind_arena_free(struct memkind *kind, void *ptr)
@@ -543,43 +516,43 @@ MEMKIND_EXPORT void memkind_arena_free(struct memkind *kind, void *ptr)
     if (kind == MEMKIND_DEFAULT) {
         jemk_free(ptr);
     } else if (ptr != NULL) {
+        pthread_once(&kind->init_once, kind->ops->init_once);
         jemk_dallocx(ptr, get_tcache_flag(kind->partition, 0));
     }
 }
 
 MEMKIND_EXPORT void memkind_arena_free_with_kind_detect(void *ptr)
 {
-    struct memkind *kind = memkind_arena_detect_kind(ptr);
-
-    memkind_arena_free(kind, ptr);
-}
-
-MEMKIND_EXPORT size_t memkind_arena_malloc_usable_size(void *ptr)
-{
-    return memkind_default_malloc_usable_size(NULL, ptr);
+    memkind_arena_free(memkind_arena_detect_kind(ptr), ptr);
 }
 
 MEMKIND_EXPORT void *memkind_arena_realloc(struct memkind *kind, void *ptr,
                                            size_t size)
 {
+    pthread_once(&kind->init_once, kind->ops->init_once);
     unsigned arena;
 
     if (size == 0 && ptr != NULL) {
-        memkind_arena_free(kind, ptr);
-        ptr = NULL;
+        jemk_dallocx(ptr, get_tcache_flag(kind->partition, 0));
     } else {
         int err = kind->ops->get_arena(kind, &arena, size);
         if (MEMKIND_LIKELY(!err)) {
             if (ptr == NULL) {
-                ptr = jemk_mallocx_check(size,
-                                         MALLOCX_ARENA(arena) | get_tcache_flag(kind->partition, size));
+                return jemk_mallocx_check(
+                    size,
+                    MALLOCX_ARENA(arena) |
+                        get_tcache_flag(kind->partition, size));
             } else {
-                ptr = jemk_rallocx_check(ptr, size,
-                                         MALLOCX_ARENA(arena) | get_tcache_flag(kind->partition, size));
+                ptr = jemk_rallocx(ptr, size,
+                                   MALLOCX_ARENA(arena) |
+                                       get_tcache_flag(kind->partition, size));
+                if (MEMKIND_UNLIKELY(!ptr))
+                    errno = ENOMEM;
+                return ptr;
             }
         }
     }
-    return ptr;
+    return NULL;
 }
 
 int memkind_arena_update_cached_stats(void)
@@ -598,18 +571,18 @@ MEMKIND_EXPORT void *memkind_arena_realloc_with_kind_detect(void *ptr,
     struct memkind *kind = memkind_arena_detect_kind(ptr);
     if (kind == MEMKIND_DEFAULT) {
         return memkind_default_realloc(kind, ptr, size);
-    } else {
-        return memkind_arena_realloc(kind, ptr, size);
     }
+    return memkind_arena_realloc(kind, ptr, size);
 }
 
-MEMKIND_EXPORT int memkind_arena_update_memory_usage_policy(
-    struct memkind *kind, memkind_mem_usage_policy policy)
+MEMKIND_EXPORT int
+memkind_arena_update_memory_usage_policy(struct memkind *kind,
+                                         memkind_mem_usage_policy policy)
 {
     int err = MEMKIND_SUCCESS;
     unsigned i;
     ssize_t dirty_decay_val = 0;
-    switch ( policy ) {
+    switch (policy) {
         case MEMKIND_MEM_USAGE_POLICY_DEFAULT:
             dirty_decay_val = DIRTY_DECAY_MS_DEFAULT;
             break;
@@ -624,9 +597,11 @@ MEMKIND_EXPORT int memkind_arena_update_memory_usage_policy(
     for (i = 0; i < kind->arena_map_len; ++i) {
         char cmd[64];
 
-        snprintf(cmd, sizeof(cmd), "arena.%u.dirty_decay_ms", kind->arena_zero + i);
-        err = jemk_mallctl(cmd, NULL, NULL, (void *)&dirty_decay_val, sizeof(ssize_t));
-        if ( err ) {
+        snprintf(cmd, sizeof(cmd), "arena.%u.dirty_decay_ms",
+                 kind->arena_zero + i);
+        err = jemk_mallctl(cmd, NULL, NULL, (void *)&dirty_decay_val,
+                           sizeof(ssize_t));
+        if (err) {
             log_err("Incorrect dirty_decay_ms value %zu", dirty_decay_val);
             return MEMKIND_ERROR_INVALID;
         }
@@ -638,15 +613,16 @@ MEMKIND_EXPORT int memkind_arena_update_memory_usage_policy(
 MEMKIND_EXPORT void *memkind_arena_calloc(struct memkind *kind, size_t num,
                                           size_t size)
 {
-    void *result = NULL;
+    pthread_once(&kind->init_once, kind->ops->init_once);
     unsigned arena;
 
     int err = kind->ops->get_arena(kind, &arena, size);
     if (MEMKIND_LIKELY(!err)) {
-        result = jemk_mallocx_check(num * size,
-                                    MALLOCX_ARENA(arena) | MALLOCX_ZERO | get_tcache_flag(kind->partition, size));
+        return jemk_mallocx_check(num * size,
+                                  MALLOCX_ARENA(arena) | MALLOCX_ZERO |
+                                      get_tcache_flag(kind->partition, size));
     }
-    return result;
+    return NULL;
 }
 
 MEMKIND_EXPORT int memkind_arena_posix_memalign(struct memkind *kind,
@@ -655,6 +631,7 @@ MEMKIND_EXPORT int memkind_arena_posix_memalign(struct memkind *kind,
 {
     int err;
     unsigned arena;
+    pthread_once(&kind->init_once, kind->ops->init_once);
 
     *memptr = NULL;
     err = kind->ops->get_arena(kind, &arena, size);
@@ -668,9 +645,10 @@ MEMKIND_EXPORT int memkind_arena_posix_memalign(struct memkind *kind,
         /* posix_memalign should not change errno.
            Set it to its previous value after calling jemalloc */
         int errno_before = errno;
-        *memptr = jemk_mallocx_check(size,
-                                     MALLOCX_ALIGN(alignment) | MALLOCX_ARENA(arena) | get_tcache_flag(
-                                         kind->partition, size));
+        *memptr =
+            jemk_mallocx_check(size,
+                               MALLOCX_ALIGN(alignment) | MALLOCX_ARENA(arena) |
+                                   get_tcache_flag(kind->partition, size));
         errno = errno_before;
         err = *memptr ? 0 : ENOMEM;
     }
@@ -684,6 +662,15 @@ MEMKIND_EXPORT int memkind_bijective_get_arena(struct memkind *kind,
     return 0;
 }
 
+// SplitMix64 hash
+static uint64_t hash64(uint64_t x)
+{
+    x += 0x9e3779b97f4a7c15;
+    x = (x ^ (x >> 30)) * 0xbf58476d1ce4e5b9;
+    x = (x ^ (x >> 27)) * 0x94d049bb133111eb;
+    return x ^ (x >> 31);
+}
+
 #ifdef MEMKIND_TLS
 MEMKIND_EXPORT int memkind_thread_get_arena(struct memkind *kind,
                                             unsigned int *arena, size_t size)
@@ -693,20 +680,16 @@ MEMKIND_EXPORT int memkind_thread_get_arena(struct memkind *kind,
     arena_tsd = pthread_getspecific(kind->arena_key);
 
     if (MEMKIND_UNLIKELY(arena_tsd == NULL)) {
-        arena_tsd = malloc(sizeof(unsigned int));
+        arena_tsd = jemk_malloc(sizeof(unsigned int));
         if (arena_tsd == NULL) {
             err = MEMKIND_ERROR_MALLOC;
             log_err("malloc() failed.");
         }
         if (!err) {
-            // On glibc pthread_self() is incremented by 0x801000 for every
-            // thread (no matter the arch's word width).  This might change
-            // in the future, but even in the worst case the hash will
-            // degenerate to a single bucket with no loss of correctness.
-            *arena_tsd = ((uint64_t)pthread_self() >> 12) %
-                         kind->arena_map_len;
-            err = pthread_setspecific(kind->arena_key, arena_tsd) ?
-                  MEMKIND_ERROR_RUNTIME : 0;
+            *arena_tsd = hash64((uint64_t)pthread_self()) % kind->arena_map_len;
+            err = pthread_setspecific(kind->arena_key, arena_tsd)
+                ? MEMKIND_ERROR_RUNTIME
+                : 0;
         }
     }
     *arena = kind->arena_zero + *arena_tsd;
@@ -734,57 +717,31 @@ MEMKIND_EXPORT int memkind_thread_get_arena(struct memkind *kind,
                                             unsigned int *arena, size_t size)
 {
     unsigned int arena_idx;
-    // it's likely that each thread control block lies on different page
-    // so we extracting page number with >> 12 to improve hashing
-    arena_idx = (get_fs_base() >> 12) & kind->arena_map_mask;
+    arena_idx = hash64(get_fs_base()) & kind->arena_map_mask;
     *arena = kind->arena_zero + arena_idx;
     return 0;
 }
-#endif //MEMKIND_TLS
+#endif // MEMKIND_TLS
 
 static void *jemk_mallocx_check(size_t size, int flags)
 {
-    /*
-     * Checking for out of range size due to unhandled error in
-     * jemk_mallocx().  Size invalid for the range
-     * LLONG_MAX <= size <= ULLONG_MAX
-     * which is the result of passing a negative signed number as size
-     */
-    void *result = NULL;
-
-    if (MEMKIND_UNLIKELY(size >= LLONG_MAX)) {
-        errno = ENOMEM;
-    } else if (size != 0) {
-        result = jemk_mallocx(size, flags);
+    if (MEMKIND_LIKELY(size)) {
+        void *ptr = jemk_mallocx(size, flags);
+        if (MEMKIND_UNLIKELY(!ptr))
+            errno = ENOMEM;
+        return ptr;
     }
-    return result;
-}
-
-static void *jemk_rallocx_check(void *ptr, size_t size, int flags)
-{
-    /*
-     * Checking for out of range size due to unhandled error in
-     * jemk_mallocx().  Size invalid for the range
-     * LLONG_MAX <= size <= ULLONG_MAX
-     * which is the result of passing a negative signed number as size
-     */
-    void *result = NULL;
-
-    if (MEMKIND_UNLIKELY(size >= LLONG_MAX)) {
-        errno = ENOMEM;
-    } else {
-        result = jemk_rallocx(ptr, size, flags);
-    }
-    return result;
-
+    return NULL;
 }
 
 void memkind_arena_init(struct memkind *kind)
 {
     if (kind != MEMKIND_DEFAULT) {
-        int err = memkind_arena_create_map(kind, get_extent_hooks_by_kind(kind));
+        int err =
+            memkind_arena_create_map(kind, get_extent_hooks_by_kind(kind));
         if (err) {
-            log_fatal("[%s] Failed to create arena map (error code:%d).", kind->name, err);
+            log_fatal("[%s] Failed to create arena map (error code:%d).",
+                      kind->name, err);
             abort();
         }
     }
@@ -805,7 +762,7 @@ static int memkind_arena_get_stat(struct memkind *kind, memkind_stat_type stat,
             bool is_init;
             size_t sz_b_state = sizeof(is_init);
             snprintf(cmd, 128, "arena.%u.initialized", kind->arena_zero + i);
-            err =  jemk_mallctl(cmd, (void *)&is_init, &sz_b_state, NULL, 0);
+            err = jemk_mallctl(cmd, (void *)&is_init, &sz_b_state, NULL, 0);
             if (err) {
                 log_err("Error on getting initialized state of arena.");
                 return MEMKIND_ERROR_INVALID;
@@ -816,7 +773,8 @@ static int memkind_arena_get_stat(struct memkind *kind, memkind_stat_type stat,
         }
 
         for (j = 0; j < arena_stats[stat].stats_no; ++j) {
-            snprintf(cmd, 128, arena_stats[stat].stats[j], kind->arena_zero + i);
+            snprintf(cmd, 128, arena_stats[stat].stats[j],
+                     kind->arena_zero + i);
             err = jemk_mallctl(cmd, &temp_stat, &sz, NULL, 0);
             if (err) {
                 log_err("Error on getting arena statistic.");
@@ -829,7 +787,8 @@ static int memkind_arena_get_stat(struct memkind *kind, memkind_stat_type stat,
 }
 
 int memkind_arena_get_stat_with_check_init(struct memkind *kind,
-                                           memkind_stat_type stat, bool check_init, size_t *value)
+                                           memkind_stat_type stat,
+                                           bool check_init, size_t *value)
 {
     int status;
     switch (stat) {
@@ -842,7 +801,7 @@ int memkind_arena_get_stat_with_check_init(struct memkind *kind,
             *value = PAGE_2_BYTES(*value);
             break;
         default:
-            //not reached
+            // not reached
             return MEMKIND_ERROR_INVALID;
             break;
     }
@@ -866,10 +825,9 @@ int memkind_arena_get_global_stat(memkind_stat_type stat, size_t *value)
     return err;
 }
 
-int memkind_arena_enable_background_threads(size_t threads_limit)
+int memkind_arena_set_max_bg_threads(size_t threads_limit)
 {
-    bool background_thread_val = true;
-    int err;
+    int err = MEMKIND_ERROR_INVALID;
 
     if (threads_limit) {
         err = jemk_mallctl("max_background_threads", NULL, NULL, &threads_limit,
@@ -879,27 +837,21 @@ int memkind_arena_enable_background_threads(size_t threads_limit)
             return MEMKIND_ERROR_INVALID;
         }
     }
-    err = jemk_mallctl("background_thread", NULL, NULL, &background_thread_val,
-                       sizeof(bool));
+    return err;
+}
+
+int memkind_arena_set_bg_threads(bool state)
+{
+    int err =
+        jemk_mallctl("background_thread", NULL, NULL, &state, sizeof(bool));
     if (err) {
-        log_err("Error on activating background thread");
+        log_err("Error while enabling/disabling background thread");
         return MEMKIND_ERROR_INVALID;
     }
     return err;
 }
 
-#define DEST_SLAB_END(begin, size) ((uintptr_t)begin+ (uintptr_t)size)
-
-struct mem_util_stats {
-    void *target_slab;      // address of the slab of a potential realloaction would go to ( NULL in case of large/huge allocation)
-    size_t nfree;           // number of free regions in the slab
-    size_t nregs;           // number of regions in the slab
-    size_t slab_size;       // size of the slab in bytes
-    size_t bin_nfree;       // total number of free regions in the bin the slab belongs to
-    size_t bin_nregs;       // total number of regions in the bin the slab belongs to
-};
-
-void *memkind_arena_defrag_reallocate_with_kind_detect (void *ptr)
+void *memkind_arena_defrag_reallocate_with_kind_detect(void *ptr)
 {
     return memkind_arena_defrag_reallocate(memkind_detect_kind(ptr), ptr);
 }
@@ -910,29 +862,74 @@ void *memkind_arena_defrag_reallocate(struct memkind *kind, void *ptr)
         return NULL;
     }
 
-    size_t out_sz = sizeof(struct mem_util_stats);
-    struct mem_util_stats out;
-    int err = jemk_mallctl("experimental.utilization.query", &out, &out_sz, &ptr,
-                           sizeof(ptr));
-    if (err) {
-        log_err("Error on get utilization query");
-        return NULL;
-    }
-
-    // Check if input pointer resides outside of potential reallocation slab
-    // Check if occupied regions inside the slab are below average occupied regions inside bin
-    // Check if there are some free regions in the destination slab
-    if (out.target_slab &&
-        ((ptr < out.target_slab) ||
-         (uintptr_t)ptr > DEST_SLAB_END(out.target_slab, out.slab_size)) &&
-        out.nfree * out.bin_nregs >= out.nregs * out.bin_nfree &&
-        out.nfree != 0) {
-        size_t size = memkind_malloc_usable_size(kind, ptr);
+    if (!jemk_check_reallocatex(ptr)) {
+        size_t size = jemk_malloc_usable_size(ptr);
         void *ptr_new = memkind_arena_malloc_no_tcache(kind, size);
-        if (MEMKIND_UNLIKELY(!ptr_new)) return NULL;
+        if (MEMKIND_UNLIKELY(!ptr_new))
+            return NULL;
         memcpy(ptr_new, ptr, size);
-        memkind_free(kind, ptr);
+        jemk_dallocx(ptr, MALLOCX_TCACHE_NONE);
         return ptr_new;
     }
     return NULL;
 }
+
+static bool is_stats_print_opts_valid(memkind_stat_print_opt opts)
+{
+    CLEAR_BIT(opts, MEMKIND_STAT_PRINT_JSON_FORMAT);
+    CLEAR_BIT(opts, MEMKIND_STAT_PRINT_OMIT_GENERAL);
+    CLEAR_BIT(opts, MEMKIND_STAT_PRINT_OMIT_MERGED_ARENA);
+    CLEAR_BIT(opts, MEMKIND_STAT_PRINT_OMIT_DESTROYED_MERGED_ARENA);
+    CLEAR_BIT(opts, MEMKIND_STAT_PRINT_OMIT_PER_ARENA);
+    CLEAR_BIT(opts, MEMKIND_STAT_PRINT_OMIT_PER_SIZE_CLASS_BINS);
+    CLEAR_BIT(opts, MEMKIND_STAT_PRINT_OMIT_PER_SIZE_CLASS_LARGE);
+    CLEAR_BIT(opts, MEMKIND_STAT_PRINT_OMIT_MUTEX);
+    CLEAR_BIT(opts, MEMKIND_STAT_PRINT_OMIT_EXTENT);
+
+    return opts ? false : true;
+}
+
+int memkind_arena_stats_print(void (*write_cb)(void *, const char *),
+                              void *cbopaque, memkind_stat_print_opt opts)
+{
+    if (!is_stats_print_opts_valid(opts)) {
+        log_err("Printing statistics failed. Unsupported options '%d'", opts);
+        return MEMKIND_ERROR_INVALID;
+    }
+
+    char temp[12] = "";
+    char *char_opts = NULL;
+    if (opts != MEMKIND_STAT_PRINT_ALL) {
+        if (opts & MEMKIND_STAT_PRINT_JSON_FORMAT) {
+            strcat(temp, "J");
+        }
+        if (opts & MEMKIND_STAT_PRINT_OMIT_GENERAL) {
+            strcat(temp, "g");
+        }
+        if (opts & MEMKIND_STAT_PRINT_OMIT_MERGED_ARENA) {
+            strcat(temp, "m");
+        }
+        if (opts & MEMKIND_STAT_PRINT_OMIT_DESTROYED_MERGED_ARENA) {
+            strcat(temp, "d");
+        }
+        if (opts & MEMKIND_STAT_PRINT_OMIT_PER_ARENA) {
+            strcat(temp, "a");
+        }
+        if (opts & MEMKIND_STAT_PRINT_OMIT_PER_SIZE_CLASS_BINS) {
+            strcat(temp, "b");
+        }
+        if (opts & MEMKIND_STAT_PRINT_OMIT_PER_SIZE_CLASS_LARGE) {
+            strcat(temp, "l");
+        }
+        if (opts & MEMKIND_STAT_PRINT_OMIT_MUTEX) {
+            strcat(temp, "x");
+        }
+        if (opts & MEMKIND_STAT_PRINT_OMIT_EXTENT) {
+            strcat(temp, "e");
+        }
+        char_opts = temp;
+    }
+
+    jemk_malloc_stats_print(write_cb, cbopaque, char_opts);
+    return MEMKIND_SUCCESS;
+}
